# -*- coding: utf-8 -*-
import numpy as np
from balu.FeatureTransformation import Bft_sigmoid


def Bcl_outscore(ds, score, options):

    outp = 0
    if 'output' in options:
        outp = options['output']

    if outp == 1:
        ds = score
    elif outp == 2:
        ds = np.concatenate((ds.ravel(), score.ravel()))
    elif outp == 3:
        ds = np.concatenate((ds.ravel(), (Bft_sigmoid(score, options['param']) - 0.5).ravel() * 2))

    return ds



# -*- coding: utf-8 -*-
import numpy as np


def Bft_norm(X, normtype):
    """ Xnew, a, b = Bft_norm(X, normtype)

     Toolbox: Balu

        Normalization of features X.
        normtype = 1 for variance = 1 and mean = 0
        normtype = 0 for max = 1, min = 0

        Xnew = a*X + b

     Example:
        from balu.ImagesAndData import balu_load
        from matplotlib.pyplot import figure
        from balu.InputOutput import Bio_plotfeatures
        from balu.FeatureTransformation import Bft_norm

        data = balu_load('datareal')
        f, d = data['f'], data['d']
        X = f[:, 0:2]
        figure(1); Bio_plotfeatures(X, d)
        Xnew, a, b = Bft_norm(X, 0)
        figure(2); Bio_plotfeatures(Xnew, d)

     (c) Grima, PUC-DCC, 2010: D. Mery, E.Cortazar
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    N, M = X.shape
    if normtype == 1:
        mf = np.mean(X, axis=0)
        sf = np.std(X, axis=0)
        a = np.ones((1, M)) / sf
        b = -mf / sf
    else:
        mi = np.min(X, axis=0)
        ma = np.max(X, axis=0)
        md = ma - mi + (ma == mi).astype(int)
        a = np.ones((1, M)) / md
        b = -mi / md

    Xnew = X * np.dot(np.ones((N, 1)), a) + np.ones((N, 1)) * b

    return Xnew, a, b


# -*- coding: utf-8 -*-
import numpy as np


def Bft_sigmoid(sc, d, show=False):
    """ function [param]=fitSigmoid2Scores(posFeats, negFeats, classifier, display)
     original de A.Soto
     nPosExamples=size(posFeats,1);
     nNegExamples=size(negFeats,1);

     [predict_label, accuracy, posScores]=svmpredict31EchoOff(ones(nPosExamples,1), posFeats, classifier);
     [predict_label, accuracy, negScores]=svmpredict31EchoOff(ones(nNegExamples,1), negFeats, classifier);

     Modificado por D.Mery

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    '''if d.size > 2:
        dneg = np.amin(d);
        dpos = np.amax(d);

        posScores = sc(d == dpos);
        negScores = sc(d == dneg);

        nPosExamples = length(posScores);
        nNegExamples = length(negScores);

        x = [negScores;
        posScores];

        y = [zeros(nNegExamples, 1);
        ones(nPosExamples, 1)];

        param = nlinfit(x, y, @ fsig, [-1 0.05]);'''


    return 0


# -*- coding: utf-8 -*-
import numpy as np


def Bfa_sp100(X, d):
    """ Sp = Bfa_sp100(X, d)

     Toolbox: Balu
        Especificty at Sensibility = 100%.
        X features matrix. X(i,j) is the feature j of sample i.
        d vector that indicates the ideal classification of the samples

     See also Bfs_sfs, Bfa_fisher

     (c) D.Mery, PUC-DCC, 2011
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if len(d.shape) == 1:
        d = np.expand_dims(d, axis=1)

    N, m = X.shape

    d = d - d.min()
    if d.max() > 1:
        print('Bfa_sp100 works for only two classes')
        exit()

    d1, d2 = np.where(d == 1)

    minz = np.min(X[d1, :], axis=0)
    maxz = np.max(X[d1, :], axis=0)
    z1 = np.zeros(X.shape)
    for p in range(m):
        ii = np.where(np.logical_and(X[:, p] >= minz[p], X[:, p] <= maxz[p]))[0]
        z1[ii, p] = np.ones((ii.size))

    if m > 1:
       drs = np.sum(z1, axis=1)
       ii = np.where(drs == X.shape[1])[0]
       dr = np.zeros(d.shape)
       dr[ii] = np.ones((ii.size, 1))
    else:
       dr = z1


    TP = np.sum(dr*d, axis=0)
    FP = np.sum(dr, axis=0) - TP

    TN = np.sum((1 - dr) * (1 - d), axis=0)
    Sp = (TN/(FP+TN))[0]
    return Sp



# -*- coding: utf-8 -*-
import numpy as np
from balu.InputOutput import Bio_statusbar
from balu.FeatureAnalysis import Bfa_score
import matplotlib.pyplot as plt


def Bfs_sfs(X, d, options={'show': True}, hold=False):
    """ selec = Bfs_sfs(X, d, options)

     Toolbox: Balu
        Sequential Forward Selection for fatures X according to ideal
        classification d. optins.m features will be selected.
        options['b']['method'] = 'fisher' uses Fisher objetctive function.
        options['b']['method'] = 'sp100' uses as criteria Sp @Sn=100%.
        options['b] can be any Balu classifier structure (see example).
        options['show'] = 1 display results.
        selec is the indices of the selected features.
        hold Keeps the bar graph opens until the user close it. Only
             works if show is true in the options

     Example 1: SFS using Fisher dicriminant
        data = balu_load('datareal')
        f = data['f']
        d = data['d']
        fn = data['fn']
        op = {
            'm': 10,                            # 10 features will be selected
            'show': True,                       # display results
            'b': {'name': 'fisher'}             # SFS with Fisher
        }
        s = Bfs_sfs(f, d, op)                   # index of selected features
        X = f[:, s]                             # selected features
        Xn = fn[s]                              # list of feature names
        op_lda = {'p': np.array([])}
        ds, op_lda = Bcl_lda(X, d, X, op_lda)   # LDA classifier
        p = Bev_performance(d, ds)              # performance with sfs
        print p

     Example 2: SFS using a KNN classifier
        from balu.ImagesAndData import balu_load
        from balu.FeatureSelection import Bfs_sfs

        data = balu_load('datareal')
        f = data['f']
        d = data['d']
        fn = data['fn']
        op = {
            'm': 10,                            # 10 features will be selected
            'show': True,                       # display results
            'b': {'name': 'knn',                # SFS with KNN with 5 neigbors
                  'options': {
                      'k': 5
                  }
            }
        }
        s = Bfs_sfs(f, d, op)                   # index of selected features
        X = f[:, s]                             # selected features
        Xn = fn[s]                              # list of feature names

     Example 3: SFS using sp100 criterion
        from balu.ImagesAndData import balu_load
        from balu.FeatureSelection import Bfs_sfs

        data = balu_load('datareal')
        f = data['f']
        d = data['d']
        fn = data['fn']
        op = {
            'm': 10,                            # 10 features will be selected
            'show': True,                       # display results
            'b': {'name': 'sp100'}              # SFS with sp100 criterion
        }
        s = Bfs_sfs(f, d, op)                   # index of selected features
        X = f[:, s]                             # selected features
        Xn = fn[s]                              # list of feature names

     (c) D.Mery, PUC-DCC, Jul. 2011
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    plt.ion()
    m = options['m']
    show = options['show']

    if 'force' not in options:
        options['force'] = False
    force = options['force']

    f = X;
    N = f.shape[1]
    selec = np.array([])              #selected features
    J = np.zeros((m, 1))
    # Jmax = 0
    k = 0

    if show:
        ff = Bio_statusbar('SFS progress')

    while k < m:

        fnew = False
        Jmax = float(np.finfo(np.float32).min)
        for i in range(N):
            if k == 0 or np.sum(selec == i) == 0:
                s = np.hstack((selec, i))
                fs = f[:, s.astype(int)]
                Js = Bfa_score(fs, d, options)
                if Js >= Jmax:
                    ks = i
                    Jmax = Js
                    fnew = True

        if fnew:
            selec = np.hstack((selec, ks))
            J[k] = Jmax
            k += 1

            if show:
                ff = Bio_statusbar(k / float(m), ff)
                plt.clf()
                plt.title('Selected features')
                plt.bar(np.arange(m) + 0.5, J)

                print(', Jmax = {0:8.4f}\n'.format(Jmax))
                for i in range(selec.size):
                    plt.text(i+0.4, J[i]*1.05, '{0}'.format(selec[i]))

                plt.show(block=False)
                plt.pause(0.001)

        else:
            print('Bfs_sfs: no more improvement. Sequential search for feature selection is interrupted.')
            if force and k < m:
               print('Bfs_sfs: Warning! {0} random features were selected in order to have'.format(m-k))
               print('                  {0} selected features (options.force is true).\n'.format(m))
               t = np.arange(N)
               t = np.delete(t, selec)
               n = t.size
               x = np.random.rand(n)
               i = np.sort(x)
               selec = np.hstack((selec, t[i[0:m-k+1]]))

            k = float(np.finfo(np.float32).max)

    if show and hold:
        plt.pause(-1)
        #delete(ff)
    plt.ioff()

    return selec.astype(int)


# -*- coding: utf-8 -*-
import numpy as np


def Bds_gaussgen(m, s, n):
    """ X, d = Bds_gaussgen(m, s, n)

     Toolbox: Balu

        Gaussian Random Sample Generator.
        m matrix qxp. m(i,j) is mean of class i for feature j
        s matrix qxp. s(i,j) is std of class i for feature j
        n vector qx1. n(i) is number of samples of class i

        Example for two classes and two features:
            import numpy as np
            from balu.InputOutput import Bio_plotfeatures
            from balu.DataSelectionAndGeneration import Bds_gaussgen

            mu = np.array([[10, 1], [1, 10]])
            st = 4 * np.ones((2, 2))
            X, d = Bds_gaussgen(mu, st, 500 * np.ones((2, 1)))
            Bio_plotfeatures(X, d)

        Example for three classes and two features:
            import numpy as np
            from balu.InputOutput import Bio_plotfeatures
            from balu.DataSelectionAndGeneration import Bds_gaussgen

            mu = np.array([[2, 1], [1, 2], [2, 2]])
            st = np.ones((3, 2)) / 4.0
            X, d = Bds_gaussgen(mu, st, 500 * np.ones((3, 1)))
            Bio_plotfeatures(X, d)

     (c) GRIMA-DCCUC, 2011
     http://grima.ing.puc.cl

    With collaboration from:
    Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    m = np.array(m)
    s = np.array(s)
    n = np.array(n)

    if len(n.shape) > 1:
        n = np.squeeze(n)

    n = n.astype(int)

    q = n.size                 # number of classes
    p = m.shape[1]             # number of features
    N = np.sum(n)              # number of samples
    X = np.zeros((N, p))
    d = np.zeros((N, 1))
    t = 0

    for i in range(q):
        d[t:t + n[i]] = i + 1
        x = np.zeros((n[i], p))
        for j in range(p):
            x[:, j] = np.random.normal(m[i, j], s[i, j], n[i])

        X[t:t + n[i], :] = x
        t = t + n[i]

    return X, d



# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore
from sklearn.neighbors import KDTree
from scipy.stats import mode


def Bcl_knn(*args):
    """ ds, options = Bcl_knn(X, d, Xt, options)  Training & Testing together
     options = Bcl_knn(X, d, options)     Training only
     ds, _ = Bcl_knn(Xt, options)      Testing only

     Toolbox: Balu
        KNN (k-nearest neighbors) classifier using randomized kd-tree
        forest from FLANN. This implementation requires scikit-learn.

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X
           options.k is the number of neighbors (default=10)

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['kdtree'] contains information about the randomized kdtree
           (from KDTree function of scikit-learn).
           options['string'] is a 8 character string that describes the performed
           classification (e.g., 'knn,10  ' means knn with k=10).

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_knn
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'k': 10}
            ds, _ = Bcl_knn(X, d, Xt, op)           # knn with 10 neighbors
            p = Bev_performance(ds, dt)             # performance on test data
            print p

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_knn

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'k': 10}
            op = Bcl_knn(X, d, op)                  # knn with 10 neighbors

        Example: Testing only (after training only example):
            ds, _ = Bcl_knn(Xt, op)                 # knn with 10 neighbors - testing
            p = Bev_performance(ds, dt)             # performance on test data


     D.Mery, C. Mena PUC-DCC, 2010-2013
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    train, test, X, d, Xt, options = Bcl_construct(args)
    options = options.copy()
    options['string'] = 'knn,{0:2d}  '.format(options['k'])

    if train:
        options['kdtree'] = KDTree(X, metric='euclidean')
        #options['X'] = X
        if len(d.shape) < 2:
            options['d'] = d[:, None]
        else:
            options['d'] = d
        output = options

    if test:
        kdtree = options['kdtree']
        dist, i = kdtree.query(Xt, k=options['k'], return_distance=True)

        if options['k'] > 1:
            ds = mode(options['d'][i.T][:, :, 0]).mode.T
        else: # for 1-KNN (modification by Carlos Mera)
            ds = options['d'][i.T][:, :, 0].T

        if 'output' in options:
            ns = ds.size
            sc = np.zeros((ns, 1))
            for q in range(ns):
                j, _ = np.where(options['d'][i.T[:, q]] == ds[q])
                sc[q] = np.min(dist.T[j, q]**2)

            ds = Bcl_outscore(ds, sc, options)

        output = ds, options

    return output


# -*- coding: utf-8 -*-
import numpy as np
import sys


def Bio_statusbar(*args):
    if len(args) == 1:
        s = args[0]
        if s is str:
            return s
        else:
            return 'Progress ...'
    elif len(args) == 2:
        s = args[0]
        if s is str:
            return s
        else:
            v = args[0]
            s = args[1]
            point = int(20*v)
            sys.stdout.write(
                "\r[" + "=" * point + " " * (20 - point) + "] " + str(int(v * 100)) + "% ")
            sys.stdout.flush()



# -*- coding: utf-8 -*-
import numpy as np


def Bfx_flusser(R, options={}):
    """ X, Xn = Bfx_flusser(R, options)
     X, Xn = Bfx_flusser(R)

     Toolbox: Balu

        Extract the four Flusser moments from binary image R.

        options.show = 1 display mesagges.

        X is a 4 elements vector:
          X(i): Flusser-moment i for i=1,...,4.
        Xn is the list of feature names.

        Reference:
        Sonka et al. (1998): Image Processing, Analysis, and Machine Vision,
        PWS Publishing. Pacific Grove, Ca, 2nd Edition.

        Example:
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from skimage.morphology import label
            from balu.FeatureExtraction import Bfx_flusser

            I = balu_imageload('testimg3.jpg')           # input image
            R, _, _ = Bim_segbalu(I)                           # segmentation
            L = label(R)                                 # regions
            n = int(L.max())
            imshow(L)
            show()
            X = np.array([])
            for i in range(n):
                Xi, Xn = Bfx_flusser(L == i + 1)         # Flusser moments
                if X.size == 0:
                    X = Xi
                else:
                    X = np.vstack((X, Xi))

            print X

       See also Bfx_standard, Bfx_hugeo, Bfx_fitellipse, Bfx_gupta.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if 'show' not in options:
        options['show'] = False

    if options['show']:
        print('--- extracting Flusser moments...')

    Ireg, Jreg = np.where(R == 1)           # pixels in the region

    i_m = np.mean(Ireg)
    j_m = np.mean(Jreg)
    A = Ireg.size
    I0 = np.ones((A, 1))
    J0 = np.ones((A, 1))
    I1 = Ireg[:, None] - i_m * np.ones((A, 1))
    J1 = Jreg[:, None] - j_m * np.ones((A, 1))
    I2 = I1 * I1
    J2 = J1 * J1
    I3 = I2 * I1
    J3 = J2 * J1

    # Central moments
    u00 = np.squeeze(np.dot(I0.T, J0))
    # u01 = np.dot(I0.T, J1); not used
    u02 = np.squeeze(np.dot(I0.T, J2))
    u03 = np.squeeze(np.dot(I0.T, J3))
    # u10 = np.dot(I1.T, J0); not used
    u20 = np.squeeze(np.dot(I2.T, J0))
    u30 = np.squeeze(np.dot(I3.T, J0))
    u11 = np.squeeze(np.dot(I1.T, J1))
    u12 = np.squeeze(np.dot(I1.T, J2))
    u21 = np.squeeze(np.dot(I2.T, J1))

    II1 = (u20*u02-u11**2)/u00**4
    II2 = (u30**2*u03**2-6*u30*u21*u12*u03+4*u30*u12**3+4*u21**3*u03-3*u21**2*u12**2)/u00**10
    II3 = (u20*(u21*u03-u12**2)-u11*(u30*u03-u21*u12)+u02*(u30*u12-u21**2))/u00**7
    II4 = (u20**3*u03**2-6*u20**2*u11*u12*u03-6*u20**2*u02*u21*u03+9*u20**2*u02*u12**2 + 12*u20*u11**2*u21*u03+6*u20*u11*u02*u30*u03-18*u20*u11*u02*u21*u12-8*u11**3*u30*u03- 6*u20*u02**2*u30*u12+9*u20*u02**2*u21+12*u11**2*u02*u30*u12-6*u11*u02**2*u30*u21+u02**3*u30**2)/u00**11

    X  = np.array([[II1, II2, II3, II4]])

    Xn = [ 'Flusser-moment 1        ',
           'Flusser-moment 2        ',
           'Flusser-moment 3        ',
           'Flusser-moment 4        ']
    return X, Xn



# -*- coding: utf-8 -*-
from skimage.filters import threshold_otsu
from .Bim_rgb2hcm import Bim_rgb2hcm
from .Bim_morphoreg import Bim_morphoreg


def Bim_segbalu(I, p=-0.05):
    """ R, E, J = Bim_segbalu(I, p)

     Toolbox: Balu
      Segmentation of an object with homogeneous background.

      I: input image
      p: threshold (default: p=-0.05) with p between -1 and 1.
          A positive value is used to dilate the segmentation,
          the negative to erode.
      R: binary image of the object
      E: binary image of the edge of the object
      J: high contrast image of I.

      See details in:
      Mery, D.; Pedreschi, F. (2005): Segmentation of Colour Food Images using
      a Robust Algorithm. Journal of Food Engineering 66(3): 353-360.

      Example:
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from matplotlib.pyplot import figure, imshow, title, show

            I = balu_imageload('testimg1.jpg')
            R, E, J = Bim_segbalu(I)
            figure(1)
            imshow(I), title('test image')
            figure(2)
            imshow(R, cmap='gray'), title('segmented image')
            show()

         Repeat this examples for images testimg2, testimg3 and testimg4. Last
         test image requires R = Bim_segbalu(I,-0.1) for better results.

     See also Bim_segmowgli, Bim_segotsu, Bim_segkmeans, Bio_segshow.

     D.Mery, PUC-DCC, Apr. 2008-2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    J = Bim_rgb2hcm(I.astype(float) / 256.0)
    t = threshold_otsu(J)
    R, E = Bim_morphoreg(J, t+p)
    return R, E, J



# -*- coding: utf-8 -*-
import numpy as np


def Bds_nostratify(X, d, s):
    """ X1, d1, X2, d2 = Bds_nostratify(X, d, s)

     Toolbox: Balu

        Data Sampling without Stratification (without replacement)

        input: (X,d) means features and ideal classification
        Bnostratify takes randomily a portion s (s between 0 and 1) of the
        whole that without considering the portion of each class
        from (X,d) to build (X1,d1). The samples not used in (X1,d1) are
        stored in (X2,d2).

     D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """
    if len(d.shape) < 2:
        d = d[:, None]

    N = X.shape[0]
    rn = np.random.rand(N)
    j = np.argsort(rn)
    Xr = X[j, :]
    dr = d[j, 0]
    r = np.floor(s * N)
    R = np.array([[0, r], [r, N]]).astype(int)
    X1 = Xr[R[0, 0]:R[0, 1], :]
    d1 = dr[R[0, 0]:R[0, 1]]
    X2 = Xr[R[1, 0]:R[1, 1], :]
    d2 = dr[R[1, 0]:R[1, 1]]

    return X1, d1, X2, d2


# -*- coding: utf-8 -*-
import numpy as np
import warnings


def Bfs_clean(X, show=False):
    """select = Bfs_clean(X,show)

     Toolbox: Balu
        Feature selection cleaning.

        It eliminates constant features and correlated features.

        Input: X is the feature matrix.
               show = 1 displays results (default show=0)
        Output: selec is the indices of the selected features

     Example:

        from balu.FeatureSelection import Bfs_clean
        from balu.ImagesAndData import balu_load

        k = balu_load('datareal')
        s = Bfs_clean(f, 1)             # index of selected features
        X = f[:, s]                     # selected features
        Xn = fn[s]                      # list of feature names
        print 'Original:', f.shape      # original set of features
        print 'Selected:', X.shape      # set of selecte features

     D.Mery, PUC-DCC, Jul. 2009
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    f = X

    nf = f.shape[1]
    p = np.arange(0, nf)
    ip = np.zeros((nf, 1))

    # eliminating correlated features
    warnings.filterwarnings('ignore')
    if f.shape[0] == 1:
        return p
    else:
        C = np.abs(np.corrcoef(f, rowvar=0))
    ii, jj = np.where(C > 0.99)

    if ii.size > 0:
        for i in range(ii.size):
            if np.abs(ii[i] - jj[i]) > 0:
                k = max(ii[i], jj[i])
                t, = np.where(p == k)
                n = p.size
                if t.size > 0:
                    if t[0] == 1:
                        p = p[1:]
                    else:
                        if t[0] == n - 1:
                            p = p[:n - 1]
                        else:
                            # p = p[[1:t - 1  t + 1:n]];
                            p = np.delete(p, t)

    ip[p] = 1

    # eliminating constant features
    s = np.std(f, axis=0)
    ii, = np.where(s < 1e-8)
    if ii.size > 0:
        ip[ii] = 0

    p, _ = np.where(ip)
    fc = f[:, p]
    nc = fc.shape[1]
    if show:
        print('Bfs_clean: number of features reduced from {0} to {1}.\n'.format(nf, nc))

    return p



# -*- coding: utf-8 -*-
import numpy as np
from skimage.measure import find_contours


def Bfx_fourierdes(R, options={}):
    """ X, Xn = Bfx_fourierdes(R, options)

     Toolbox: Balu
        Computes the Fourier descriptors of a binary image R.

        options['show'] = True display messages.
        options['Nfourierdes'] number of descriptors.

        X is the feature vector
        Xn is the list of feature names.

        Reference:
        Zahn, C; Roskies, R.: Fourier Descriptors for Plane
        Closed Curves, IEEE Trans on Computers, C21(3):269-281, 1972

       Example:
            from balu.ImagesAndData import balu_imageload
            from balu.FeatureExtraction import Bfx_fourierdes
            from balu.InputOutput import Bio_printfeatures
            from balu.ImageProcessing import Bim_segbalu

            I = balu_imageload('testimg1.jpg')      # input image
            R = Bim_segbalu(I)                      # segmentation
            X, Xn = Bfx_fourierdes(R)               # Fourier descriptors
            Bio_printfeatures(X, Xn)

       See also Bfx_fitellipse, Bfx_hugeo, Bfx_gupta, Bfx_flusser.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if len(options) == 0:
        options['show'] = False
        options['Nfourierdes'] = 16

    if options['show']:
        print('--- extracting Fourier descriptors...')

    N = options['Nfourierdes']

    jj = 1j

    B = find_contours(R, 0.9, positive_orientation='high')
    g = B[0]

    V = g[:, 1] + jj * g[:, 0]
    m = g.shape[0]

    r = np.zeros((m, 1)).astype(complex)
    phi = np.zeros((m, 1))
    dphi = np.zeros((m, 1))
    l = np.zeros((m, 1))
    dl = np.zeros((m, 1))

    r[0] = V[0] - V[m - 1]
    for i in range(1, m):
        r[i] = V[i] - V[i - 1]

    for i in range(m):
        dl[i] = np.abs(r[i])
        phi[i] = np.angle(r[i])

    for i in range(m - 1):
        dphi[i] = np.mod(phi[i + 1] - phi[i] + np.pi, 2 * np.pi) - np.pi

    dphi[m - 1] = np.mod(phi[0] - phi[m - 1] + np.pi, 2 * np.pi) - np.pi

    for k in range(m):
        l[k] = 0
        for i in range(k + 1):
            l[k] = l[k] + dl[i]

    L = l[m - 1]

    A = np.zeros((N, 1))

    for n in range(1, N + 1):
        an = 0
        bn = 0
        for k in range(m):
            an = an + dphi[k] * np.sin(2 * np.pi * n * l[k] / L)
            bn = bn + dphi[k] * np.cos(2 * np.pi * n * l[k] / L)

        an = -an / n / np.pi
        bn = bn / n / np.pi
        imagi = an + jj * bn
        A[n - 1] = np.abs(imagi)

    X = A.T
    Xn = N * ['']
    for i in range(N):
        Xn[i] = 'Fourier-des {0}'.format(i)

    return X, Xn



# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from sklearn.neural_network import MLPClassifier


def Bcl_nn(*args):
    """ ds, options = Bcl_nn(X, d, Xt, op)  Training & Testing together
     options = Bcl_nn(X, d, op)     Training only
     ds, options = Bcl_nn(Xt, options) Testing only

     Toolbox: Balu
        Neural Network using scikit-learn's MLPClassifier.

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X
           options['method'] = 0, 1, 2, 3 for 'identity','tanh' or 'relu' (default=3)
           options['iter'] is the max. number of iterations used in the MLPClassifier.fit algorithm
           (default=100).

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['net'] contains information about the neural network
           (from class MLPClassifier in scikit-learn).
           options['dmin'] contains np.min(d).
           options['string'] is a 8 character string that describes the performed
           classification (e.g., 'nn,3 ' means relu - neural network).

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_nn
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'method': 3, 'iter': 12}
            ds, _ = Bcl_nn(X, d, Xt, op)            # logistic - neural network
            p = Bev_performance(ds, dt)             # performance on test data

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_nn

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'method': 3, 'iter': 12}
            op = Bcl_nn(X, d, op)                   # logistic - neural network - training

        Example: Testing only (after training only example):
            ds, _ = Bcl_nn(Xt, op)                  # logistic - neural network - testing
            p = Bev_performance(ds, dt)             # performance on test data


        Implementation based on Bcl_nnlgm from Balu Matlab toolbox and Neural Network function
        from scikit-learn.

     D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     And

     D. Patiño (dapatinoco@unal.edu.co) (2016)
     https://github.com/dipaco
     """

    train, test, X, d, Xt, options = Bcl_construct(args)

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    options = options.copy()

    if 'iter' not in options:
        options['iter'] = 100

    options['string'] = 'nn,{0} '.format(options['method'])
    if train:

        all_activation_functions = ['identity', 'logistic', 'tanh', 'relu']
        m = 'relu'
        c = options['method']
        if isinstance(c, int):
            if 0 <= c < len(all_activation_functions):
                m = all_activation_functions[c]
        elif c in all_activation_functions:
            m = c

        dmin = d.min()

        net = MLPClassifier(solver='lbfgs', activation=m, max_iter=options['iter'])
        net.fit(X, d.ravel())
        options['net'] = net
        options['dmin'] = dmin
        output = options

    if test:
        net = options['net']
        ds = net.predict(Xt)
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import numpy as np


def Bio_printfeatures(X, *args):
    """
     Bio_printfeatures(X, Xn)    # for features with feature names
     Bio_printfeatures(X)        # for feature values only

     Toolbox: Balu
        Display extracted features.
        Xn: feature names  (matrix mxp, one row per each string of p
            characters)
        X:  feature values (vector 1xm for m features)
        Xu: feature units  (matrix mxq, one row per each string of q
            characters)

        These variables are the outputs of Bgeofeatures or Bintfeatures.

        The output of Bio_printfeatures is like this:

        1 center of grav i       [pixels]         163.297106
        2 center of grav j       [pixels]         179.841850
        3 Height                 [pixels]         194.000000
        4 Width                  [pixels]         196.000000
        5 Area                   [pixels]         29361.375000
        :         :           :                  :

       Example 1: Display of standard geometric features of testimg1.jpg
          I = imread('testimg1.jpg');            # input image
          R = Bsegbalu(I);                       # segmentation
          [X,Xn,Xu] = Bfg_standard(R);           # standard geometric features
          Bprintfeatures(X,Xn,Xu)

       Example 2: Display of first 5 samples of datagauss.mat
          load datagauss
          Xn = ['[length]';'[weigh] '];
          Xu = ['cm';'kg'];
          for i=1:5
             fprintf('Sample %d:\n',i);
             Bprintfeatures(X(i,:),Xn,Xu)
             Benterpause
          end


       See also Bplotfeatures.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    N = np.max(X.shape)
    if len(args) > 0:
        Xn = args[0]
    else:
        Xn = N * ['']

    for k in range(len(Xn)):
        print('{0} {1} {2}'.format(k, Xn[k], np.squeeze(X)[k]))


# -*- coding: utf-8 -*-
from scipy.io import loadmat
from os.path import dirname, realpath, join
from scipy.ndimage import imread

__all__ = ['balu_load', 'balu_imageload']


def balu_load(name):
    """ data = balu_load(data_filename)     #without extension

       Example:
          from balu.ImagesAndData import balu_load
          data = balu_load('datareal')         # data filename

    (c) Diego Patiño (dapatinoco@unal.edu.co) (2016)
    """
    dir_path = dirname(realpath(__file__))
    return loadmat(join(dir_path, name + '.mat'))


def balu_imageload(name):
    """ I = balu_imageload(image_filename)     #Image filename

       Example:
          from balu.ImagesAndData import balu_imageload
          I = balu_imageload(image_filename)         # Image filename

    (c) Diego Patiño (dapatinoco@unal.edu.co) (2016)
    """
    dir_path = dirname(realpath(__file__))
    return imread(join(dir_path, name))



from .Bfx_geo import Bfx_geo
from .Bfx_fourier import Bfx_fourier
from .Bfx_all import Bfx_all
from .Bfx_basicgeo import Bfx_basicgeo
from .Bfx_hugeo import Bfx_hugeo
from .Bfx_fourierdes import Bfx_fourierdes
from .Bfx_geo import Bfx_geo
from .Bfx_fitellipse import Bfx_fitellipse
from .Bfx_flusser import Bfx_flusser
from .Bfx_haralick import Bfx_haralick

__all__ = ['Bfx_geo', 'Bfx_fourier', 'Bfx_all', 'Bfx_basicgeo', 'Bfx_hugeo', 'Bfx_fourierdes', 'Bfx_geo',
           'Bfx_fitellipse', 'Bfx_flusser', 'Bfx_haralick', 'Bfx_lbp']



# -*- coding: utf-8 -*-
import numpy as np


def Bfx_haralick(I, R=None, options={}):
    """ X, Xn = Bfx_haralick(I, R, options)
     X, Xn = Bfx_haralick(I, options)
    
     Toolbox: Balu
        Haralick texture features.
    
        X is a 28 elements vector with mean and range of mean and range of

         1 Angular Second Moment
         2 Contrast
         3 Correlacion
         4 Sum of squares
         5 Inverse Difference Moment
         6 Sum Average
         8 Sum Entropy
         7 Sum Variance
         9 Entropy
        10 Difference Variance
        11 Difference Entropy
        12,13 Information Measures of Correlation
        14 Maximal Correlation Coefficient

        Xn is the list of name features.

        I is the image. R is the binary image that indicates which pixels of I will be
        computed.
        options['dharalick'] is the distance in pixels used to compute the
        coocurrence matrix.
        options['show'] = True display results.

        Reference:
        Haralick (1979): Statistical and Structural Approaches to Texture,
        Proc. IEEE, 67(5):786-804

       Example 1: only one distance (3 pixels)
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from balu.FeatureExtraction import Bfx_haralick
            from balu.InputOutput import Bio_printfeatures

            options = {'dharalick': 3}              #3 pixels distance for coocurrence
            I = balu_imageload('testimg1.jpg')      #input image
            R, _, _ = Bim_segbalu(I)                #segmentation
            J = I[:, :, 1]                          #green channel
            X, Xn = Bfx_haralick(J, R, options)     #Haralick features
            Bio_printfeatures(X, Xn)

       Example 2: five distances (1,2,...5 pixels)
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from balu.FeatureExtraction import Bfx_haralick
            from balu.InputOutput import Bio_printfeatures

            options = {'dharalick': [1, 2, 3, 4, 5]}         #3 and 5 pixels distance for coocurrence
            I = balu_imageload('testimg1.jpg')      #input image
            R, _, _ = Bim_segbalu(I)                #segmentation
            J = I[:, :, 1]                          #green channel
            X, Xn = Bfx_haralick(J, R, options)     #Haralick features
            Bio_printfeatures(X, Xn)

       See also Bfx_gabor, Bfx_clp, Bfx_fourier, Bfx_dct, Bfx_lbp.

     (c) GRIMA-DCCUC, 2011
     http://grima.ing.puc.cl
    
     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    I = I.astype(float)

    if R is None:
        R = np.ones(I.shape)

    dseq = options['dharalick']

    if 'show' not in options:
        options['show'] = False

    if options['show']:
        print('--- extracting Haralick texture features...')

    if isinstance(dseq, (list, np.dtype)):
        dseq = np.array(dseq)
    else:
        dseq = np.array([dseq])

    m = dseq.size
    n = 28 * m

    X = np.zeros((1, n))
    Xn = n * [None]
    k = 0
    for i in range(m):
        d = dseq[i]

        Cd000 = Bcoocurrencematrix(I, R, d, 0) + Bcoocurrencematrix(I, R, -d, 0)
        Cd000 = Cd000 / np.sum(Cd000)
        Cd045 = Bcoocurrencematrix(I, R, d, -d) + Bcoocurrencematrix(I, R, -d, d)
        Cd045 = Cd045 / np.sum(Cd045)
        Cd090 = Bcoocurrencematrix(I, R, 0, d) + Bcoocurrencematrix(I, R, 0, -d)
        Cd090 = Cd090 / np.sum(Cd090)
        Cd135 = Bcoocurrencematrix(I, R, d, d) + Bcoocurrencematrix(I, R, -d, -d)
        Cd135 = Cd135 / np.sum(Cd135)

        TexMat = np.hstack((Bcoocurrencefeatures(Cd000), Bcoocurrencefeatures(Cd045), Bcoocurrencefeatures(Cd090), Bcoocurrencefeatures(Cd135)))
        X[0, i*28:(i+1)*28] = np.hstack((np.mean(TexMat, axis=1), np.max(np.abs(TexMat.T), axis=0)))

        for q in range(2):
            if q == 0:
                sq = 'mean '
            else:
                sq = 'range'

            for s in range(14):
                Xn[k] = 'Tx{0:2d},d{1:2d}({2})         '.format(s, d, sq)
                k += 1

    return X, Xn


def Bcoocurrencematrix(I, R, Io, Jo):
    """P = Bcoocurrencematrix(I, R, Io, Jo)

     Coocurrence matrix of the pixels of image I indicated by binary image R
     following the direction (Io,Jo).

     (c) D.Mery, PUC-DCC, Apr. 2008

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    V = np.floor(I / 32.0)
    N, M = I.shape
    Z1 = np.zeros((N+40, M+40))
    Z2 = Z1.copy()
    R1 = Z1.copy()
    R2 = R1.copy()
    Z1[14:N+14, 14:M+14] = V
    Z2[14+Io:N+14+Io, 14+Jo:M+14+Jo] = V
    R1[14:N+14, 14:M+14] = R
    R2[14+Io:N+14+Io, 14+Jo:M+14+Jo] = R
    ii, jj = np.where(np.logical_not(np.logical_and(R1, R2)))
    Z1[ii, jj] = -1
    Z2[ii, jj] = -1
    T1 = Z1.ravel()
    T2 = Z2.ravel()
    d = np.where(np.logical_and((T1 > -1), (T2 > -1)))[0]

    if d.size > 0:
        P = np.zeros((8, 8))
        T = np.vstack((T1[d], T2[d])).T
        I = np.lexsort((T2[d], T1[d]))
        X = T[I, :]
        ne = X.shape[0]
        i1 = np.where(np.logical_or(
            (np.insert(X[:, 0], 0, -1) - np.insert(X[:, 0], ne, -1)) != 0,
            (np.insert(X[:, 1], 0, -1) - np.insert(X[:, 1], ne, -1)) != 0
        ))[0]
        i2 = np.insert(i1[1:], i1.size - 1, -1)
        d = i2 - i1
        for i in range(d.size - 1):
            P[int(X[i1[i], 1]), int(X[i1[i], 0])] = d[i]

    else:
        P = -np.ones((8, 8))

    return P


def Bcoocurrencefeatures(P):
    """ Tx = Bcoocurrencefeatures(P)

     Haralick texture features calculated from coocurrence matrix P.

     (c) D.Mery, PUC-DCC, Apr. 2008

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    Pij = P.ravel()[:, None]
    Ng = 8
    pxi = np.sum(P, axis=1)[:, None]
    pyj = np.sum(P, axis=0)[:, None]

    ux = np.mean(pxi, axis=0)
    uy = np.mean(pyj, axis=0)

    sx = np.std(pxi, axis=0)
    sy = np.std(pyj, axis=0)

    pxy1 = np.zeros((2*Ng-1, 1))
    for k in range(1, 2*Ng):
        s = 0
        for i in range(Ng):
            for j in range(Ng):
                if i + j == k - 1:
                    s += P[i, j]

        pxy1[k - 1] = s

    pxy2 = np.zeros((Ng, 1))
    for k in range(Ng):
        s = 0
        for i in range(Ng):
            for j in range(Ng):
                if np.abs(i-j) == k:
                    s += P[i, j]

        pxy2[k] = s

    Q = np.zeros((Ng, Ng))
    pxi += 1e-20
    pyj += 1e-20

    for i in range(Ng):
        for j in range(Ng):
            s = 0
            for k in range(Ng):
                s += P[i, k] * P[j, k] / pxi[i] / pyj[k]

            Q[i, j] = s

    eigQ = np.linalg.eig(Q)[0]

    i, j = np.where(P >= 0)
    dif = i - j
    dif2 = dif * dif
    dif21 = dif2 + 1


    # 1 Angular Second Moment
    f1 = np.dot(Pij.T, Pij)[0][0]

    # 2 Contrast
    f2 = np.dot(np.arange(Ng)[None]**2, pxy2)[0][0]

    # 3 Correlacion
    f3 = (np.sum(i*j*np.squeeze(Pij)) - ux*uy*Ng**2)/sx/sy

    # 4 Sum of squares
    f4 = np.dot(dif2[None], Pij)[0][0]

    # 5 Inverse Difference Moment
    f5 = np.sum(Pij / dif21[:, None])

    # 6 Sum Average
    f6 = np.dot(np.arange(2, 2*Ng + 1)[None], pxy1)[0][0]

    # 8 Sum Entropy
    #f8 = -pxy1'*log(pxy1+1e-20);
    f8 = np.dot(-pxy1.T, np.log(pxy1 + 1e-20))[0][0]

    # 7 Sum Variance
    if8 = np.arange(2, 2*Ng + 1)[None] - f8
    f7 = np.dot(if8, pxy1)[0][0]

    # 9 Entropy
    f9 = np.dot(-Pij.T, np.log(Pij+1e-20))[0][0]

    # 10 Difference Variance
    f10 = np.var(pxy2, axis=0)

    # 11 Difference Entropy
    f11 = np.dot(-pxy2.T, np.log(pxy2+1e-20))[0][0]

    # 12,13 Information Measures of Correlation
    HXY = f9
    pxipyj = pxi[i] * pyj[j]
    HXY1 = np.dot(-Pij.T, np.log(pxipyj + 1e-20))[0][0]
    HXY2 = np.dot(-pxipyj.T, np.log(pxipyj+1e-20))
    HX = np.dot(-pxi.T, np.log(pxi + 1e-20))[0][0]
    HY = np.dot(-pyj.T, np.log(pyj + 1e-20))[0][0]
    f12 = ((HXY-HXY1) / max(HX, HY))
    f13 = (1 - np.exp(-2 * (HXY2-HXY)))

    # 14 Maximal Corrleation Coefficient
    f14 = eigQ[1]

    Tx = np.array([[f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14]]).T

    return Tx



from .Bfa_jfisher import Bfa_jfisher
from .Bfa_score import Bfa_score
from .Bfa_mutualinfo import Bfa_mutualinfo
from .Bfa_relevance import Bfa_relevance
from .Bfa_mRMR import Bfa_mRMR
from .Bfa_sp100 import Bfa_sp100

__all__ = ['Bfa_jfisher', 'Bfa_score', 'Bfa_mutualinfo', 'Bfa_relevance', 'Bfa_mRMR', 'Bfa_sp100']



# -*- coding: utf-8 -*-
import numpy as np
from .Bev_confusion import Bev_confusion


def Bev_performance(d1, d2, nn=None):
    """p = Bev_erformance(d1,d2,nn)

     Toolbox: Balu
        Performance evaluation between two classifications, e.g., ideal (d1)
        and real (d2) classification.

        d1 and d2 are vectors or matrices (vector Nxn1 and Nxn2 respectivelly)
        at least n1 or n2 muts be one. N is the number of samples.
        p is the performance, diagonal sum divided by N.
        The classes should be labeled as 1, 2, ... n, but
        if nn = [i j] is given it indicates that the lowest
        class is min(nn) and the highest class is max(nn).
        Thus, n = max(nn)-min(nn)+1 and the size of T is nxn.

        d1 (or d2) can be a matrix (Nxn1 or Nxn2) containing the classification
        of n1 (or n2) classifiers. In this case the output p is a vector
        n1x1 (or n2x1) containing the performance of each classifier.
        If n1=1 and n2=1, then p is a scalar.

        Bperformance(d1,d2) and Bperformance(d2,d1) obtain the same result.

        Example:
           load datagauss             % simulated data (2 classes, 2 features)
           Bio_plotfeatures(X,d)      % plot feature space
           op.p = [];
           ds1 = Bcl_lda(X,d,Xt,op);  % LDA classifier
           ds2 = Bcl_qda(X,d,Xt,op);  % QDA classifier
           ds = [ds1 ds2];
           p = Bev_performance(ds,dt) % performance on test data

     D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    d1 = d1.astype(float)
    d2 = d2.astype(float)

    if len(d1.shape) < 2:
        d1 = np.expand_dims(d1, axis=1)
    n1 = d1.shape[1]

    if len(d2.shape) < 2:
        d2 = np.expand_dims(d2, axis=1)
    n2 = d2.shape[1]

    if n1 == 1 or n2 == 1:

        if n2 > n1:
            ds = d2
            d = d1
        else:
            d = d2
            ds = d1

        n = ds.shape[1]
        p = np.zeros((n, 1))
        for i in range(n):
            if nn is not None:
                T, pp = Bev_confusion(d, ds[:, i], nn)
            else:
                T, pp = Bev_confusion(d, ds[:, i])
            p[i] = np.mean(pp)

    else:
        print('Bev_performance: at least d1 or d2 must have only one column')

    if p.size == 1:
        return p[0, 0]
    else:
        return p[:, 0]


from .Bcl_lda import Bcl_lda
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore
from .Bcl_structure import Bcl_structure
from .Bcl_knn import Bcl_knn
from .Bcl_maha import Bcl_maha
from .Bcl_qda import Bcl_qda
from .Bcl_dmin import Bcl_dmin
from .Bcl_svm import Bcl_svm
from .Bcl_nn import Bcl_nn

__all__ = ['Bcl_lda', 'Bcl_construct', 'Bcl_outscore', 'Bcl_lda', 'Bcl_structure', 'Bcl_knn', 'Bcl_maha',
           'Bcl_qda', 'Bcl_dmin', 'Bcl_svm', 'Bcl_nn']



# -*- coding: utf-8 -*-
import numpy as np
import warnings
from matplotlib.pyplot import xlabel, ylabel, plot, show, legend, title, text, scatter, gcf, subplot, locator_params, clf
from mpl_toolkits.mplot3d import Axes3D


def Bio_plotfeatures(X, d, Xn=[], hold=True):
    """Bio_plotfeatures(X, d, Xn)

     Toolbox: Balu

     Plot features X according classification d. If the feature names are
     given in Xn then they will labeled in each axis.

     For only one feature, histograms are ploted.
     For two( or three) features, plots in 2D( or 3D) are given.
     For m > 3 features, m x m 2D plots are given(feature i vs.feature j)

     Example 1: 1D & 2D
        from balu.InputOutput import Bio_plotfeatures
        from balu.ImagesAndData import balu_load
        from matplotlib.pyplot import figure

        data = balu_load('datagauss')          # simulated data(2 classes, 2 features)
        X = data['X']
        d = data['d']
        figure(1)
        Bio_plotfeatures(X[:, 0], d, 'x1')     # histogram of feature 1
        figure(2)
        Bio_plotfeatures(X, d)                 # plot feature space in 2D(2 features)

     Example 2: 3D
        from balu.InputOutput import Bio_plotfeatures
        from balu.ImagesAndData import balu_load

        data = balu_load('datareal')          # real data
        f = data['f']
        d = data['d']
        X = f[:, [220, 174, 234]]             # only three features are choosen
        Bio_plotfeatures(X, d)                # plot feature space in 3D(3 features)

     Example 3: 5D(using feature selection)
         from balu.InputOutput import Bio_plotfeatures
         from balu.ImagesAndData import balu_load

         data = balu_load('datareal ')             # real data
         f = data['f']
         d = data['d']
         op['m'] = 5;                              # 5 features will be selected
         op['s'] = 0.75;                           # only 75 % of sample will be used
         op['show'] = False                        # display results
         op['b']['name'] = 'fisher';               # definition SFS with Fisher
         s = Bfs_balu(f, d, op);                   # feature selection
         Bio_plotfeatures(f[:, s], d)              # plot feature space for 5 features

     See also Bev_roc.

     (c)
     GRIMA - DCCUC, 2011
     http: // grima.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    clf()

    if len(X.shape) > 1:
        m = X.shape[1]
    else:
        m = 1

    if m > 9:
        print('Bio_plotfeatures for {0} features makes {1} plots.'.format(m, m * m))
        exit()

    scflag = False
    if len(d.shape) > 1:
        if d.shape[1] == 2:
            sc = d[:, 1]
            d = d[:, 0]
            scflag = True
        else:
            d = np.squeeze(d)

    dmin = int(np.min(d))
    dmax = int(np.max(d))

    col = 'gbrcmykbgrcmykbgrcmykbgrcmykgbrcmykbgrcmykbgrcmykbgrcmykgbrcmykbgrcmykbgrcmykbgrcmyk'
    mar = 'ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^ox+v^'

    # clf
    warnings.filterwarnings('ignore')
    s = []
    for k in range(dmin, dmax + 1):
        s.append('class {0}'.format(k - 1))

    if m < 4:
        if m == 1:
            for k in range(dmin, dmax + 1):
                idx = np.where(d == k)
                h, x = np.histogram(X[idx], 100)
                dx = x[1] - x[0]
                A = np.sum(h) * dx
                x = np.hstack((x - dx / 2.0, x[-1] + dx))
                h = np.hstack((0, h, 0))
                plot(x, h / A, color=col[k])

            if len(Xn) != 0:
                xlabel(Xn)
            else:
                xlabel('feature value')

            ylabel('PDF')
        elif m == 2:
            if len(Xn) == 0:
                Xn = ['feature value 1', 'feature value 2']

            for k in range(dmin, dmax + 1):
                ii = np.where(d == k)
                scatter(X[ii, 0], X[ii, 1], c=col[k], marker=mar[k])
                if scflag:
                    for ic in range(1, ii[0].size):
                        text(X[ii[0][ic], 0], X[ii[0][ic], 1], '  ' + str(sc[ii[0][ic]]))

            title('feature space')
            xlabel(Xn[0])
            ylabel(Xn[1])
        elif m == 3:

            fig = gcf()
            ax = fig.add_subplot(111, projection='3d')

            for k in range(dmin, dmax + 1):
                ii = np.where(d == k)
                ax.scatter(X[ii, 0], X[ii, 1], X[ii, 2], c=col[k], marker=mar[k])

                if scflag:
                    for ic in range(1, ii[0].size):
                        ax.text(X[ii[0][ic], 0], X[ii[0][ic], 1], X[ii[0][ic], 2], '  ' + str(sc[ii[0][ic]]))

            if len(Xn) > 0:
                ax.set_xlabel(Xn[0])
                ax.set_ylabel(Xn[1])
                ax.set_zlabel(Xn[2])
            else:
                ax.set_xlabel('feature value 1')
                ax.set_ylabel('feature value 2')
                ax.set_zlabel('feature value 3')
        legend(s)
    else:

        l = 1
        for j in range(1, m + 1):
            for i in range(1, m + 1):
                zi = X[:, i - 1]
                zj = X[:, j - 1]
                subplot(m, m, l)
                l += 1

                for k in range(dmin, dmax + 1):
                    ii = np.where(d == k)
                    scatter(zi[ii], zj[ii], c=col[k], marker=mar[k])
                    locator_params(nbins=3)

                    if scflag:
                        for ic in range(1, ii[0].size):
                            text(zi[ii[0][ic]], zj[ii[0][ic]], '  ' + str(sc[ii[0][ic]]))

                if len(Xn) > 0:
                    xl = Xn[i - 1]
                    yl = Xn[j - 1]
                else:
                    xl = 'z_{0}'.format(i - 1)
                    yl = 'z_{0}'.format(j - 1)

                if i == 1:
                    ylabel(yl)

                if j == m:
                    xlabel(xl)

    if hold:
        show()



# -*- coding: utf-8 -*-
import numpy as np
from warnings import filterwarnings

def Bfa_jfisher(X, d, p=None):
    """ J = Bfa_jfisher(X, d, p)

     Toolbox: Balu
        Fisher objective function J.
        X features matrix. X(i,j) is the feature j of sample i.
        d vector that indicates the ideal classification of the samples
        p a priori probability of each class

     See also Bfs_sfs.

     (c) D.Mery, PUC-DCC, 2011
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """

    if len(d.shape) == 1:
        d = np.expand_dims(d, axis=1)

    n, M = X.shape
    dmin = d.min()

    d = d - dmin + 1

    N = int(d.max())

    if p is None:
        p = np.ones((N, 1)) / float(N)

    # Centroid of all samples
    Xm = np.expand_dims(np.mean(X, axis=0), axis=0).T

    L = np.zeros((N, 1))
    Cw = np.zeros((M, M))
    Cb = np.zeros((M, M))

    for k in range(N):
        ii, jj = np.where(d == k + 1)                           # indices from class k
        L[k] = ii.size                                          # number of samples of class k
        Xk = X[ii, :]                                           # samples of class k
        Xkm = np.expand_dims(np.mean(Xk, axis=0), axis=0).T     # centroid of class k
        Ck = np.cov(Xk, rowvar=False)                           # covariance of class k

        # within-class covariance
        Cw = Cw + p[k]*Ck

        # between-class covariance
        Cb = Cb + p[k]*np.dot(Xkm - Xm, (Xkm - Xm).T)


    # Fisher discriminant
    filterwarnings('ignore')
    #print Cw
    xxx, _, _, _ = np.linalg.lstsq(Cw, Cb)
    #xxx = np.linalg.pinv(Cw)
    #J = np.trace(np.linalg.solve(Cw, Cb))
    J = np.trace(xxx)
    return J



from .Bim_segbalu import Bim_segbalu
from .Bim_segmowgli import Bim_segmowgli
from .Bim_rgb2hcm import Bim_rgb2hcm
from .Bim_morphoreg import Bim_morphoreg
from .Bim_segotsu import Bim_segotsu
from .Bim_maxmin import Bim_maxmin

__all__ = ['Bim_segbalu', 'Bim_segmowgli', 'Bim_rgb2hcm', 'Bim_morphoreg', 'Bim_segotsu', 'Bim_maxmin']


# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore


def Bcl_lda(*args):
    """ ds, options = Bcl_lda(X,d,Xt,[])  Training & Testing together
     options = Bcl_lda(X,d,[])     Training only
     ds, options = Bcl_lda(Xt,options) Testing only

     Toolbox: Balu
        LDA (linear discriminant analysis) classifier.
        We assume that the classes have a common covariance matrix

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X
           options.p is the prior probability, if p is empty,
           it will be estimated proportional to the number of samples of each
           class.

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['dmin'] contains np.min(d).
           options['Cw1'] is pinv(within-class covariance).
           options['mc'] contains the centroids of each class.
           options['string'] is a 8 character string that describes the performed
           classification (in this case 'lda     ').

        Reference:
           Hastie, T.; Tibshirani, R.; Friedman, J. (2001): The Elements of
           Statistical Learning, Springer (pages 84-90)

        Example: Training & Test together:

            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_lda
            from balu.InputOutput import Bio_plotfeatures
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           #simulated data (2 classes, 2 features)
            X = data['X']
            d = data['d']
            Xt = data['Xt']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'p': []}
            ds, options = Bcl_lda(X, d, Xt, op)     # LDA classifier
            p = Bev_performance(ds, dt)             # performance on test data
            print p

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_lda
            from balu.InputOutput import Bio_plotfeatures

            data = balu_load('datagauss')           #simulated data (2 classes, 2 features)
            X = data['X']
            d = data['d']
            Xt = data['Xt']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'p': [0.75, 0.25]}                # prior probability for each class
            op = Bcl_lda(X,d,op);                   # LDA - training

        Example: Testing only (after training only example):
            from balu.Classification import Bcl_lda

            ds, _ = Bcl_lda(Xt, op)                 # LDA - testing
            p = Bev_performance(ds, dt)             # performance on test data

        See also Bcl_qda.

     (c) GRIMA-DCCUC, 2011
     http://grima.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    train, test, X, d, Xt, options = Bcl_construct(args)
    options = options.copy()
    options['string'] = 'lda     '
    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    if train:
        dmin = np.amin(d)
        dmax = np.amax(d)
        d = d - dmin + 1
        N = d.size              # number of samples
        K = int(dmax - dmin + 1)     # number of classes

        options['p'] = np.array(options['p'])
        pest = options['p'].size == 0

        p = np.zeros(K)
        if not pest:
            p[:] = options['p']

        m = X.shape[1]
        L = np.zeros((K, 1))
        Cw = np.zeros((m, m))

        mc = np.zeros((m, K))
        for k in range(int(K)):
            ii, _ = np.where(d == k + 1)           # index of rows of class k
            if ii.size == 0:
                print('Bcl_lda: There is no class {0} in the data.'.format(k + dmin))
                exit()
            L[k, 0] = ii.size                   # number of samples in class k
            Xk = X[ii, :]                       # samples of class k
            mc[:, k] = np.mean(Xk, axis=0).T   # mean of class k
            Ck = np.cov(Xk, rowvar=False)                     # covariance of class k
            Cw = Cw + Ck * (L[k, 0] - 1)        # within-class covariance

            if pest:
                p[k] = L[k, 0] / N

        Cw /= (N - K)
        options['Cw1'] = np.linalg.pinv(Cw)
        options['dmin'] = dmin
        options['mc'] = mc
        options['p'] = p
        output = options

    if test:
        K = options['mc'].shape[1]
        Nt = Xt.shape[0]
        D = np.zeros((Nt, K))
        for k in range(int(K)):
            C1 = np.dot(options['Cw1'], options['mc'][:, k][np.newaxis].T)
            C2 = (-0.5 * np.dot(options['mc'][:, k].T, C1) + np.log(options['p'][k])) * np.ones((Nt, 1))
            D[:, k] = np.squeeze(np.dot(Xt, C1) + C2)

        a = np.amax(D, axis=1)
        i, j = np.unravel_index(np.argmax(D, axis=1), D.shape)
        sc = np.ones(a.size) / (np.abs(a) + 1e-5)
        ds = j + options['dmin']
        ds = Bcl_outscore(ds, sc, options)
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore


def Bcl_maha(*args):
    """ ds, options = Bcl_maha(X, d, Xt, None)  Training & Testing together
     options = Bcl_maha(X, d, None)     Training only
     ds, options = Bcl_maha(Xt, options) Testing only

     Toolbox: Balu
        Classifier using Mahalanobis minimal distance

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['mc'] contains the centroids of each class.
           options['dmin'] contains min(d).
           options['Ck'] is covariance matrix of each class.
           options['string'] is a 8 character string that describes the performed
           classification (in this case 'maha    ').

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_maha
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            ds, _ = Bcl_maha(X, d, Xt, None)        # Euclidean distance classifier
            p = Bev_performance(ds, dt)             # performance on test data
            print p

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_maha
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = Bcl_maha(X, d, None)               # Euclidean distance classifier

        Example: Testing only (after training only example):
            ds, _ = Bcl_maha(op, Xt)                # Euclidean distance classifier - testing
            p = Bev_performance(ds, dt)             # performance on test data

        See also Xdmin.

     D.Mery, PUC-DCC, May 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """

    train, test, X, d, Xt, options = Bcl_construct(args)

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    if options is None:
        options = {}

    options = options.copy()

    options['string'] = 'maha    '
    if train:
        m = X.shape[1]
        dmin = d.min()
        dmax = d.max()
        n = int(dmax - dmin + 1)
        d = d - dmin + 1
        mc = np.zeros((n, m))
        M = X.shape[1]
        Ck = np.zeros((M, M, n))
        for i in range(int(n)):
            ii, _ = np.where(d == i + 1)
            mc[i, :] = np.mean(X[ii, :], axis=0)
            CCk = np.cov(X[ii, :], rowvar=False)                     # covariance of class i
            Ck[:, :, i] = CCk

        options['mc'] = mc
        options['dmin'] = dmin
        options['Ck'] = Ck
        output = options

    if test:
        mc = options['mc']
        n = mc.shape[0]
        Nt = Xt.shape[0]
        ds = np.zeros((Nt, 1))
        sc = ds.copy()
        M = Xt.shape[1]

        Ck = np.zeros((M, M, n))
        for k in range(n):
            Ck[:, :, k] = np.linalg.pinv(options['Ck'][:, :, k])

        for q in range(Nt):
            dk = np.zeros((n, 1))
            for k in range(n):
                dx = Xt[q, :] - options['mc'][k, :]
                dk[k] = np.dot(dx, np.dot(Ck[:, :, k], dx.T))

            i = np.min(dk, axis=0)
            j = np.argmin(dk, axis=0)
            ds[q, 0] = j + 1
            sc[q, 0] = i

        ds = ds + options['dmin'] - 1
        ds = Bcl_outscore(ds, sc, options)
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import scipy as sp
import numpy as np
from skimage.morphology import label, disk, dilation, remove_small_objects
from skimage.color import rgb2gray
from scipy.ndimage.morphology import binary_erosion, binary_closing
from skimage.filters import laplace, gaussian
import matplotlib.pyplot as plt


def Bim_segmowgli(J, R=None, Amin=20, sig=2):
    """  F, m = Bsegmowgli(J, R, Amin, sig)

     Toolbox: Balu
      Segmentation of regions in image J using LoG edge detection.
      R   : binary image of same size of J that indicates the piexels where
            the segmentation will be performed. Default R = ones(size(J));
      Amin: minimum area of the segmented details.
      sig : sigma of LoG edge detector.
      F   : labeled image of the segmentation.
      m   : numbers of segmented regions.

      Example 1:
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segmowgli
            from matplotlib.pyplot import figure, imshow, show, title

            I = balu_imageload('rice.png')
            figure(1), imshow(I, cmap='gray'), title('test image')
            F, m = Bim_segmowgli(I, R=None, Amin=40, sig=1.5)
            figure(2), imshow(F, cmap='gray'), title('segmented image')
            show()


      Example 2:
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segmowgli, Bim_segbalu
            from matplotlib.pyplot import figure, imshow, show, title

            I = balu_imageload('testimg4.jpg')
            figure(1), imshow(I), title('test image')
            R, E, J = Bim_segbalu(I, -0.1)
            figure(2), imshow(R, cmap='gray'), title('segmented object')
            G = I[:, :, 1]
            F, m = Bim_segmowgli(G, R=R, Amin=30, sig=2)
            figure(3), imshow(F), title('segmented regions')
            show()

         Another way to display the results:
            Bio_edgeview(I,F>0)

     See also Bsegbalu.

     D.Mery, PUC-DCC, Apr. 2008
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if R is None:
        R = np.ones(J.shape, bool)

    if R.size == 0:
       R = np.ones(J.shape, bool)

    if len(J.shape) < 3:
        N, M = J.shape
        P = 1
    else:
        N, M, P = J.shape

    if P == 3:
        J = rgb2gray(J.astype(float))

    se = disk(3)
    Re = dilation(R, se)
    E = R - binary_erosion(R)

    #In the original implementation of Balu Matlab they used LoG edge detection.
    #Here we used edge detection via Canny algorithm. Then we used 0.5*sig because
    #that way we get similar results to Matlab version using edge command
    #L = canny(J, sigma=sig)
    L = edge_LoG(J, sigma=sig)
    L = np.logical_or(np.logical_and(L, Re), E)
    W = remove_small_objects(L, min_size=Amin, connectivity=2)
    F = label(np.logical_not(W), 4)
    m = int(F.max())
    for i in range(1, m + 1):
        ii, jj = np.where(F == i)
        if (ii.size < Amin) or (ii.size > N*M / 6.0):
            W[ii, jj] = 1

    F = label(np.logical_not(W), 4)
    m = int(F.max())
    print('{0} segmented regions.\n'.format(m))
    return F, m

def edge_LoG(I, sigma):
    LoG = laplace(gaussian(I, sigma=sigma), ksize=3)
    thres = np.absolute(LoG).mean() * 1.0
    output = sp.zeros(LoG.shape)
    w = output.shape[1]
    h = output.shape[0]

    for y in range(1, h - 1):
        for x in range(1, w - 1):
            patch = LoG[y - 1:y + 2, x - 1:x + 2]
            p = LoG[y, x]
            maxP = patch.max()
            minP = patch.min()
            if p > 0:
                zeroCross = True if minP < 0 else False
            else:
                zeroCross = True if maxP > 0 else False
            if ((maxP - minP) > thres) and zeroCross:
                output[y, x] = 1

    #FIXME: It is necesary to define if return the closing of the output or just the output
    #return binary_closing(output)
    return output


# -*- coding: utf-8 -*-
import numpy as np
from skimage.filters import threshold_otsu
from .Bim_morphoreg import Bim_morphoreg
from .Bim_maxmin import Bim_maxmin
from skimage.color import rgb2gray


def Bim_segotsu(I, p=0):
    """ R = Bim_segotsu(I)

     Toolbox: Balu

        Otsu segmentation of a grayvalue. This function requires Image
        Processing Toolbox.

        Input data:
           I grayvalue image.

        Output:
           R: binary image.

        Example: Training & Test together:
            from balu.ImagesAndData import balu_imageload
            from matplotlib.pyplot import figure, imshow, show, title
            from balu.ImageProcessing import Bim_segotsu

            X = balu_imageload('testimg1.jpg')
            figure(1), imshow(X), title('original image')
            R, E, J = Bim_segotsu(X)
            figure(2)
            imshow(R, cmap='gray')
            title('segmented image')
            show()

      See also Bim_segbalu, Bim_segkmeans.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    Id = I.astype(float)
    if len(I.shape) == 3 and I.shape[2] == 3:
        Id = rgb2gray(Id/256)

    J = Bim_maxmin(Id)
    n = int(J.shape[0] / 4.0)
    # Checks if the exterior border of the image is in fact part of the background
    # Otherwise it inverts the image
    if np.mean(J[0:n, 0:n]) > 0.4:
        J = 1 - J

    t = threshold_otsu(J)
    R, E = Bim_morphoreg(J, t+p)
    return R, E, J


# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore


def Bcl_qda(*args):
    """ ds, options = Bcl_qda(X, d, Xt, op)  Training & Testing together
     options = Bcl_qda(X, d, op)     Training only
     ds, options = Bcl_qda(Xt, options) Testing only

     Toolbox: Balu
        QDA (quadratic discriminant analysis) classifier.

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X
           options.p is the prior probability, if p is not given,
           it will be estimated proportional to the number of samples of each
           class.

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['dmin'] contains min(d).
           options['Ck'] is covariance matrix of each class.
           optionsmc contains the centroids of each class.
           options.string is a 8 character string that describes the performed
           classification (in this case 'qda     ').

        Reference:
           Hastie, T.; Tibshirani, R.; Friedman, J. (2001): The Elements of
           Statistical Learning, Springer (pages 84-90)

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_qda
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'p': []}
            ds, _ = Bcl_qda(X, d, Xt, op)           # QDA classifier
            p = Bev_performance(ds, dt)             # performance on test data
            print p

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_qda

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'p': [0.75, 0.25]}                # prior probability for each class
            op = Bcl_qda(X, d, op)                  # QDA - training

        Example: Testing only (after training only example):
            ds, op = Bcl_qda(Xt, op)                # QDA - testing
            p = Bev_performance(ds, dt)             # performance on test data

        See also Blda.

     (c) GRIMA, PUC-DCC 2011 - D.Mery, E.Cortazar
     http://grima.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    train, test, X, d, Xt, options = Bcl_construct(args)

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    options = options.copy()

    options['string'] = 'qda     '
    if train:
        dmin = d.min()
        dmax = d.max()
        d = d - dmin + 1
        N = d.shape[0]               # number of samples
        K = int(dmax - dmin + 1)     # number of classes

        options['p'] = np.array(options['p'])
        pest = options['p'].size == 0

        if pest:
            p = np.zeros(K)
        else:
            p = options['p']

        m = X.shape[1]
        L = np.zeros((int(K), 1))

        mc = np.zeros((m, K))
        Ck = np.zeros((m, m, K))
        for k in range(int(K)):
            ii, _ = np.where(d == k + 1)            # index of rows of class k
            if ii.size == 0:
                print('Bcl_qda: There is no class {0} in the data.'.format(k + dmin))

            L[k] = ii.size                          # number of samples in class k
            Xk = X[ii, :]                           # samples of class k
            mc[:, k] = np.mean(Xk, axis=0)          # mean of class k
            Ck[:, :, k] = np.cov(Xk, rowvar=False)  # covariance of class k
            if pest:
                p[k] = L[k] / float(N)

        options['dmin'] = dmin
        options['mc'] = mc
        options['Ck'] = Ck
        options['p'] = p
        output = options

    if test:
        K = options['mc'].shape[1]
        Nt = Xt.shape[0]
        D = np.zeros((Nt, K))
        for k in range(K):
            C = options['Ck'][:, :, k]
            Xd = (Xt - np.dot(np.ones((Nt, 1)), options['mc'][:, k][:, np.newaxis].T))
            C1 = np.diag(-0.5 * np.dot(np.dot(Xd, np.linalg.pinv(C)), Xd.T))[:, np.newaxis]
            C2 = (-0.5 * np.log(np.linalg.det(C)) + np.log(options['p'][k])) * np.ones((Nt, 1))
            D[:, k] = np.squeeze(C1 + C2)

        i = np.max(D, axis=1)
        j = np.argmax(D, axis=1)
        sc = np.ones(i.shape) / (np.abs(i) + 1e-5)
        ds = j + options['dmin']
        ds = Bcl_outscore(ds, sc, options)
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import numpy as np


def Bfx_hugeo(R, options={}):
    """[X, Xn] = Bfx_hugeo(R)
     [X, Xn] = Bfx_hugeo(R, options)

     Toolbox: Balu

     Extract the seven Hu moments from binary image R.

     options.show = 1 display mesagges.

     X is a 7 elements vector:
     X(i): Hu - moment i for i=1, ..., 7.
     Xn is the list of feature names.

     Reference:
     Hu, M - K.: "Visual Pattern Recognition by Moment Invariants",
     IRE Trans.Info.Theory IT - 8:179 - 187: 1962.

    Example:
    import numpy as np
    from balu.FeatureExtraction import Bfx_hugeo
    from balu.ImagesAndData import balu_imageload
    from balu.ImageProcessing import Bim_segbalu
    from skimage.morphology import label
    from matplotlib.pyplot import imshow, show

    I = balu_imageload('testimg3.jpg')              # input image
    R = Bim_segbalu(I)                              # segmentation
    L, n = label(R, neighbors=8, return_num=True)   # regions
    imshow(L, cmap='gray')
    X = None
    for i in range(1, n + 1):
        Xi, Xn = Bfx_hugeo(L == i)                  # Hu moments
        if X is None:
            X = Xi
        else:
            X = np.vstack((X, Xi))

    print X
    show()

     See also Bfx_basicgeo, Bfx_gupta, Bfx_fitellipse, Bfx_flusser.

    (c)
    D.Mery, PUC - DCC, 2010
    http: // dmery.ing.puc.cl

    With collaboration from:
    Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if 'show' not in options:
        options['show'] = False

    if options['show']:
        print('--- extracting Hu moments...')

    Ireg, Jreg = np.where(R.astype('uint8') == 1)         # pixels in the region
    Ireg = np.expand_dims(Ireg, axis=1)
    Jreg = np.expand_dims(Jreg, axis=1)

    i_m = np.mean(Ireg)
    j_m = np.mean(Jreg)
    A = Ireg.size
    I0 = np.ones((A, 1))
    J0 = np.ones((A, 1))
    I1 = Ireg - i_m * np.ones((A, 1))
    J1 = Jreg - j_m * np.ones((A, 1))
    I2 = I1 * I1
    J2 = J1 * J1
    I3 = I2 * I1
    J3 = J2 * J1

    # Central moments
    u00 = A             # u00 = m00 = (I0 '*J0)
    u002 = u00 * u00
    u0025 = u00 ** 2.5

    # u0015 = u00 ** 1.5 not used
    n02 = np.dot(I0.T, J2)/u002
    n20 = np.dot(I2.T, J0)/u002
    n11 = np.dot(I1.T, J1)/u002
    n12 = np.dot(I1.T, J2)/u0025
    n21 = np.dot(I2.T, J1)/u0025
    n03 = np.dot(I0.T, J3)/u0025
    n30 = np.dot(I3.T, J0)/u0025

    f1 = n20 + n02
    f2 = (n20 - n02) ** 2 + 4 * n11 ** 2
    f3 = (n30 - 3 * n12) ** 2 + (3 * n21 - n03) ** 2
    f4 = (n30 + n12) ** 2 + (n21 + n03) ** 2
    f5 = (n30 - 3 * n12) * (n30 + n12) * ((n30 + n12) ** 2 - 3 * (n21 + n03) ** 2) + (3 * n21 - n03) * (n21 + n03) * (3 * (n30 + n12) ** 2 - (n21 + n03) ** 2)
    f6 = (n20 - n02) * ((n30 + n12) ** 2 - (n21 + n03) ** 2) + 4 * n11 * (n30 + n12) * (n21 + n03)
    f7 = (3 * n21 - n03) * (n30 + n12) * ((n30 + n12) ** 2 - 3 * (n21 + n03) ** 2) - (n30 - 3 * n12) * (n21 + n03) * (3 * (n30 + n12) ** 2 - (n21 + n03) ** 2)

    X = np.zeros((1, 7))
    X[0, 0] = f1
    X[0, 1] = f2
    X[0, 2] = f3
    X[0, 3] = f4
    X[0, 4] = f5
    X[0, 5] = f6
    X[0, 6] = f7

    Xn = ['Hu-moment 1',
          'Hu-moment 2',
          'Hu-moment 3',
          'Hu-moment 4',
          'Hu-moment 5',
          'Hu-moment 6',
          'Hu-moment 7']

    return X, Xn



# -*- coding: utf-8 -*-
import numpy as np
from warnings import filterwarnings
from skimage.morphology import dilation, square
from matplotlib.pyplot import imshow, show


def Bio_edgeview(B, E, cc=None, g=1, show_now=True):
    """ Bio_edgeview(B, E, c, g)

     Toolbox: Balu
       Display gray or color image I overimposed by color pixels determined
       by binary image E. Useful to display the edges of an image.
       Variable c is the color vector [r g b] indicating the color to be displayed
      (default: c = [1 0 0], i.e., red)
       Variable g is the number of pixels of the edge lines, default g = 1

     Example to display a red edge of a food:
        from balu.ImagesAndData import balu_imageload
        from balu.ImageProcessing import Bim_segbalu
        from balu.InputOutput import Bio_edgeview

        I = balu_imageload('testimg2.jpg')             # Input image
        R, E, J = Bim_segbalu(I)                       # Segmentation
        Bio_edgeview(I, E)

     D.Mery, PUC-DCC, Apr. 2008
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    %"""

    if cc is None:
        cc = np.array([1, 0, 0])

    B = B.astype(float)

    if B.max() > 1:
        B /= 256.0

    if len(B.shape) == 2:
        N, M = B.shape
        J = np.zeros((N, M, 3))
        J[:, :, 0] = B
        J[:, :, 1] = B
        J[:, :, 2] = B
        B = J

    B1 = B[:, :, 0]
    B2 = B[:, :, 1]
    B3 = B[:, :, 2]

    Z = B1 == 0
    Z = np.logical_and(Z, B2 == 0)
    Z = np.logical_and(Z, B3 == 0)
    ii, jj = np.where(Z == 1)
    if ii.size > 0:
        B1[ii, jj] = 1 / 256.0
        B2[ii, jj] = 1 / 256.0
        B3[ii, jj] = 1 / 256.0

    filterwarnings('ignore')
    E = dilation(E, square(g))
    ii, jj = np.where(E == 1)
    B1[ii, jj] = cc[0]
    B2[ii, jj] = cc[1]
    B3[ii, jj] = cc[2]
    Y = B.astype(float)
    Y[:, :, 0] = B1
    Y[:, :, 1] = B2
    Y[:, :, 2] = B3
    imshow((255*Y).astype('uint8'))
    if show_now:
        show()



# -*- coding: utf-8 -*-
import numpy as np
from skimage.color import rgb2gray
from warnings import filterwarnings
from balu.InputOutput import Bio_edgeview
from matplotlib.widgets import RadioButtons
from matplotlib.pyplot import close, imshow, axis, draw, colorbar, gca, plot, show, title, subplot, figure


def Bio_labelregion(I, L, c):
    """ d, D = Bio_labelregion(I, L, c)

     Toolbox: Balu
        User interface to label regions of an image.

        I is the original image (color or grayvalue).
        L is a labeled image that indicates the segmented regions of I.
        c is the maximal number of classes.
        d(i) will be the class number of region i.
        D is a binary image with the corresponding labels.

     Example:
        from balu.ImagesAndData import balu_imageload
        from balu.ImageProcessing import Bim_segmowgli
        from balu.InputOutput import Bio_labelregion

        I = balu_imageload('rice.png')
        I = I[0:70, 0:70]                                   # input image
        L, m = Bim_segmowgli(I, np.ones(I.shape), 40, 1.5)  # segmented image
        d, D = Bio_labelregion(I, L, 3)
        print d

     (c) Diego Patiño dapatinoco@unal.edu.co (2016)

     Original code:
     (c) GRIMA-DCCUC, 2011
     http://grima.ing.puc.cl
    """

    close('all')
    filterwarnings('ignore')

    if len(I.shape) == 3 and I.shape[2] == 3:
        J = rgb2gray(I.astype(float))
    else:
        J = I

    figure(1)

    n = L.max()
    d = np.zeros((n, 1))
    i = 1
    D = np.zeros(J.shape)
    labels = tuple(['Correct label'] + ['class {0}'.format(a + 1) for a in range(c)])

    data = {'n': n, 'd': d, 'i': i, 'c': c, 'D': D, 'J': J, 'L': L, 'labels': labels}

    # Create the radio buttons subplot
    colorstr = '0bgrcmykwbgrcmykwbgrcmykw'
    subplot(1, 3, 1)
    title('Select the class')
    radio = RadioButtons(gca(), labels=labels)
    [cir.set_radius(cir.get_radius() * 0.5) for cir in radio.circles]
    [t.set_color(colorstr[i]) for i, t in enumerate(radio.labels)]

    subplot(1, 3, 3)
    if len(I.shape) == 3:
        imshow(I)
    else:
        imshow(I, cmap='gray')
    title('Labeled regions')

    def next_s(label):
        return mark_object(label, data)

    radio.on_clicked(next_s)
    next_s(labels[0])

    show()
    figure(1)
    subplot(1, 2, 2)
    title('Class labels')
    imshow(D)
    colorbar().set_ticks(list(range(c + 1)))
    subplot(1, 2, 1)
    Bio_edgeview(J, np.zeros(J.shape), np.array([1, 1, 0]), show_now=False)
    title('Original image')
    show()
    return d, D


def mark_object(label, data):
    colorstr = 'bgrcmykwbgrcmykwbgrcmykw'

    n = data['n']
    d = data['d']
    i = data['i']
    c = data['c']
    D = data['D']
    J = data['J']
    L = data['L']
    labels = data['labels']

    if i <= n:
        ii, jj = np.where(L == i)

        if label == labels[0]:
            i = max(i - 2, 0)
        else:
            r = labels.index(label)
            d[i - 1] = r
            D[ii, jj] = r
            x1 = jj.max() + 1
            x2 = jj.min() - 1
            y1 = ii.max() + 1
            y2 = ii.min() - 1
            subplot(1, 3, 3)
            plot(np.array([x1, x1, x2, x2, x1]), np.array([y1, y2, y2, y1, y1]), color=colorstr[r - 1])
            axis([0, J.shape[1], 0, J.shape[0]])
            gca().invert_yaxis()
            draw()

        if i == n:
            close()

        i += 1
        data['i'] = i

    subplot(1, 3, 1)
    title('Region {0}/{1}: Class label? (1... {2}):'.format(i, n, c))
    draw()

    subplot(1, 3, 2)
    R = np.zeros(J.shape)
    kk, pp = np.where(L == i)
    R[kk, pp] = 1
    title('Class label of\nyellow region?')
    Bio_edgeview(J, R, np.array([1, 1, 0]), show_now=False)
    draw()





# -*- coding: utf-8 -*-
import numpy as np
from scipy.ndimage.morphology import binary_erosion


def Bfx_fitellipse(R, options=None):
    """ X, Xn = Bfx_fitellipse(R, options)
     X, Xn = Bfx_fitellipse(R)

     Toolbox: Balu

        Fit ellipse for the boundary of a binary image R.

        options.show = 1 display mesagges.

        X is a 6 elements vector:
          X(1): Ellipse-centre i direction
          X(2): Ellipse-centre j direction
          X(3): Ellipse-minor axis
          X(4): Ellipse-major axis
          X(5): Ellipse-orientation
          X(6): Ellipse-eccentricity
          X(7): Ellipse-area

          Xn is the list of feature names.

          Xn is the list of feature names.

       Reference:
          Fitzgibbon, A.; Pilu, M. & Fisher, R.B. (1999): Direct Least Square
          Fitting Ellipses, IEEE Trans. Pattern Analysis and Machine
          Intelligence, 21(5): 476-480.

       Example:
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from balu.FeatureExtraction import Bfx_fitellipse
            from balu.InputOutput import Bio_printfeatures

            I = balu_imageload('testimg1.jpg')              # input image
            R, E, J = Bim_segbalu(I)                        # segmentation
            X, Xn = Bfx_fitellipse(R)                       # ellipse features
            Bio_printfeatures(X, Xn)

       See also Bfx_basicgeo, Bfx_hugeo, Bfx_gupta, Bfx_flusser.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    E = R - binary_erosion(R)
    Y, X = np.where(E == 1)                 # pixel of perimeter in (i,j)
    if X.size > 5:

        if options is None:
            options = {'show': False}

        if options['show']:
            print('--- extracting ellipse features...')

        # normalize data
        mx = np.mean(X)
        my = np.mean(Y)
        sx = (X.max() - X.min()) / 2.0
        sy = (Y.max() - Y.min()) / 2.0
        x = (X - mx) / sx
        y = (Y - my) / sy

        # Build design matrix
        D = np.vstack((x*x,  x*y,  y*y,  x,  y, np.ones(x.size))).T

        U, S, V = np.linalg.svd(D)
        A = V.T[:, 5]

        # unnormalize
        a = np.array([[
            A[0]*sy*sy,
            A[1]*sx*sy,
            A[2]*sx*sx,
            -2*A[0]*sy*sy*mx - A[1]*sx*sy*my + A[3]*sx*sy*sy,
            -A[1]*sx*sy*mx - 2*A[2]*sx*sx*my + A[4]*sx*sx*sy,
            A[0]*sy*sy*mx*mx + A[1]*sx*sy*mx*my + A[2]*sx*sx*my*my - A[3]*sx*sy*sy*mx - A[4]*sx*sx*sy*my + A[5]*sx*sx*sy*sy]]).T

        a = a / a[5, 0]

        # get ellipse orientation
        alpha = np.arctan2(a[1, 0], a[0, 0] - a[2, 0]) / 2.0

        # get scaled major/minor axes
        ct = np.cos(alpha)
        st = np.sin(alpha)
        ap = a[0]*ct*ct + a[1]*ct*st + a[2]*st*st
        cp = a[0]*st*st - a[1]*ct*st + a[2]*ct*ct

        # get translations
        T = np.zeros((2, 2))
        T[0, 0] = a[0, 0]
        T[0, 1] = a[1, 0] / 2.0
        T[1, 0] = a[1, 0] / 2.0
        T[1, 1] = a[2, 0]
        p = np.array([a[3], a[4]])
        mc = np.dot(-np.linalg.pinv(2*T), p)

        # get scale factor
        val = np.dot(mc.T, np.dot(T, mc))
        scale = np.abs(1.0 / (val - a[5]))

        # get major/minor axis radii
        ae = 1.0 / np.sqrt(scale * np.abs(ap))
        be = 1.0 / np.sqrt(scale * np.abs(cp))
        ecc = ae / be  # eccentricity
        ar = np.pi * ae * be

        X = np.array([[mc[0, 0], mc[1, 0], ae, be, alpha, ecc, ar]])
    else:
        X = np.array([[0., 0., 0., 0., 0., 0., 0.]])
        print('Warning: Bfx_fitellipse does not have enough points to fit')

    Xn = [
        'Ellipse-centre i [px]   ',
        'Ellipse-centre j [px]   ',
        'Ellipse-minor ax [px]   ',
        'Ellipse-major ax [px]   ',
        'Ellipse-orient [rad]    ',
        'Ellipse-eccentricity    ',
        'Ellipse-area [px]       '
        ]
    return X, Xn



# -*- coding: utf-8 -*-
import numpy as np
from .Bfa_mutualinfo import Bfa_mutualinfo
from .Bfa_relevance import Bfa_relevance
from .Bfa_mRMR import Bfa_mRMR
from .Bfa_jfisher import Bfa_jfisher
from .Bfa_sp100 import Bfa_sp100
from balu.PerformanceEvaluation import Bev_performance
from balu.Classification import Bcl_structure


def Bfa_score(X, d, options):
    """    J = Bfa_score(X, d, options)

         Toolbox: Balu
            Calculates a separability score for input data variables.
            X features matrix. X(i,j) is the feature j of sample i.
            d vector that indicates the ideal classification of the samples
            options score metric parameters

         See also Bfs_sfs.

         (c) D.Mery, PUC-DCC, 2011
         http://dmery.ing.puc.cl

         With collaboration from:
         Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """

    b = options['b']
    method = b['name'].lower()

    if method == 'mi':                                  # mutual information
        if 'param' not in b:
            dn = d.max() - d.min() + 1                  # number of classes
            p = np.ones((dn, 1)) / float(dn)
        else:
            p = b['param']

        Js = Bfa_mutualinfo(X, d, p)
    elif method == 'mr':                                # maximal relevance
        if 'param' not in b:
            dn = d.max() - d.min() + 1                  # number of classes
            p = np.ones((dn, 1)) / float(dn)
        else:
            p = b['param']

        Js = Bfa_relevance(X, d, p)
    elif method == 'mrmr':                              # minimal redundancy and maximal relevance
        if 'param' not in b:
            dn = d.max() - d.min() + 1                  # number of classes
            p = np.ones((dn, 1)) / float(dn)
        else:
            p = b['param']

        Js = Bfa_mRMR(X, d, p)
    elif method == 'fisher':                            # Fisher criterion
        if 'param' not in b:
            dn = d.max() - d.min() + 1  # number of classes
            p = np.ones((dn, 1)) / float(dn)
        else:
            p = b['param']

        Js = Bfa_jfisher(X, d, p)
    elif method == 'sp100':
        Js = Bfa_sp100(X, d)
    else:
        ds, _ = Bcl_structure(X, d, X, b)
        Js = Bev_performance(d, ds)

    return Js



from .Bio_printfeatures import Bio_printfeatures
from .Bio_plotfeatures import Bio_plotfeatures
from .Bio_edgeview import Bio_edgeview
from .Bio_labelregion import Bio_labelregion
from .Bio_statusbar import Bio_statusbar
from .Bio_decisionline import Bio_decisionline

__all__ = ['Bio_printfeatures', 'Bio_plotfeatures', 'Bio_edgeview', 'Bio_labelregion', 'Bio_statusbar', 'Bio_decisionline']



# -*- coding: utf-8 -*-
import numpy as np
from scipy.misc import imresize
from scipy.optimize import minimize


def Bim_rgb2hcm(RGB):
    """ J = Bim_rgb2hcm(RGB)

     Toolbox: Balu
        Conversion RGB to high contrast image.

        RGB: color image
        J  : hcm image

      See details in:
      Mery, D.; Pedreschi, F. (2005): Segmentation of Colour Food Images using
      a Robust Algorithm. Journal of Food Engineering 66(3): 353-360.

      Example:
         I = imread('testimg2.jpg');
         J = Bim_rgb2hcm(I);
         figure(1)
         imshow(I); title('control image')
         figure(2)
         imshow(J); title('high contrast image')

     (c) D.Mery, PUC-DCC, 2011
     http://dmery.ing.puc.cl
    """

    RGB = RGB.astype(float)
    if len(RGB.shape) < 3:
        I = RGB
    else:
        RGB64 = imresize(RGB, (64, 64), interp='bicubic')
        #k = fminsearch(@Bstdmono,[1 1],[],RGB64)

        def f(k):
            return Bstdmono(k, RGB64)

        k = minimize(f, [1, 1])['x']
        I = k[0]*RGB[:, :, 0] + k[1]*RGB[:, :, 1] + RGB[:, :, 2]

    J = I - I.min()
    J = J / J.max()
    N, M = J.shape
    n = int(min(np.floor(J.shape[0] / 4.0), N))
    m = int(min(np.floor(J.shape[1] / 4.0), M))

    if np.mean(J[0:n, 0:m]) > 0.4:
        J = 1 - J

    return J

def Bstdmono(k, RGB):
    """ s = Bstdmono(k, RGB)

     Toolbox: Balu
        Standard deviation of normalized image I, where
        I = k(1)*R+k(2)*G+B (R = RGB(:,:,1), G=RGB(:,:,2), B = RGB(:,:,3)
        s: Standard deviation

        This function is called by Brgb2hcm, Bsegbalu.

     See also Brgb2hcm, Bsegbalu.

     D.Mery, PUC-DCC, Apr. 2008
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    I = k[0] * RGB[:, :, 0] + k[1] * RGB[:,:, 1] + RGB[:, :, 2]
    s = -np.std(I) / (I.max() - I.min())
    return s


# -*- coding: utf-8 -*-
import numpy as np


def Bcl_construct(args):
    """ This function is not a classifier!!!
     This function is called by Balu classifier functions (such as Bcl_lda) to
     build the training and testing data.
    """
    train = False
    test = False
    nargin = len(args)
    if nargin == 2:
        #case 2 -> ds, options = Bcl_svm(Xt, options) # testing only
        X = np.array([])
        d = np.array([])
        Xt = args[0]
        options = args[1]
        test = True
    elif nargin == 3:
        #case 3 -> options = Bcl_svm(X, d, options) # training only
        X = args[0]
        d = args[1].astype(float)
        if X.shape[0] != d.shape[0]:
            print('Length of label vector does not match number of instances.')
            exit()

        Xt = np.array([])
        options = args[2]
        train = True
    elif nargin == 4:
        #case 4 -> ds, options = Bcl_svm(X, d, Xt, options) # training & test
        X = args[0]
        d = args[1].astype(float)
        if X.shape[0] != d.shape[0]:
            print('Length of label vector does not match number of instances.')
            exit()

        Xt = args[2]
        options = args[3]
        test = True
        train = True
    else:
        print('Bcl_construct: number of input arguments must be 2, 3 or 4.')
        exit()

    return train, test, X, d, Xt, options



# -*- coding: utf-8 -*-
import numpy as np


def Bev_confusion(d, ds, nn=None):
    """ function [T,p] = Bev_confusion(d,ds,nn);

     Toolbox: Balu
        Confusion Matrix and Performance of a classification

        d is the ideal classification (vector Nx1 with N samples)
        ds is the classified data (vector Nx1)
        T is the confusion matrix (nxn) for n classes
        T(i,j) indicates the number of samples i classified
        as j.
        p is the performance, diagonal sum divided by N
        the classes should be labeled as 1, 2, ... n, but
        if nn = [i j] is given it indicates that the lowest
        class is min(nn) and the highest class is max(nn).
        Thus, n = max(nn)-min(nn)+1 and the size of T is nxn.

     D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if nn is not None:
        n1 = np.max(nn).astype(int)
        n0 = np.min(nn).astype(int)
    else:
        n1 = np.amax(np.concatenate((d.ravel(), ds.ravel()))).astype(int)
        n0 = np.amin(np.concatenate((d.ravel(), ds.ravel()))).astype(int)

    n = n1 - n0 + 1
    T = np.zeros((n, n))
    for i in range(n0, n1 + 1):
        for j in range(n0, n1 + 1):
            kd = (d == i).astype('uint8')
            kds = (ds == j).astype('uint8')
            T[i - n0, j - n0] = np.sum(kd.ravel() * kds.ravel())

    p = np.trace(T) / np.sum(T)

    return T, p


# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from sklearn import svm


def Bcl_svm(*args):
    """ ds      = Bcl_svm(X, d, Xt, options)  Training & Testing together
     options = Bcl_svm(X, d, options)     Training only
     ds      = Bcl_svm(Xt, options)      Testing only

     Toolbox: Balu
        Support Vector Machine approach using the scikit-learn library.

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X

           options['kernel'] defines the SVM-kernel as follows:

           0: 'linear'      Linear kernel or dot product (default)
           1: 'poly'        Polynomial kernel (default order 3)
           2: 'rbf'         Gaussian Radial Basis Function kernel
           3: 'sigmoid'     Multilayer Perceptron kernel (default scale 1)

           kernel can be either int or string.

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['svmStruct'] contains information about the trained classifier
           (from SVC class of scikit-learn).
           options.string is a 8 character string that describes the performed
           classification (e.g., 'svm,4  ' means rbf-SVM).

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_svm
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'kernel': 2}
            ds, _ = Bcl_svm(X, d, Xt, op)           # rbf-SVM classifier
            p = Bev_performance(ds, dt)             # performance on test data
            print p

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_svm
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = {'kernel': 2}
            op = Bcl_svm(X, d, op)                  # rbf-SVM classifier

        Example: Testing only (after training only example):
            ds, _ = Bcl_svm(Xt, op)                 # rbf-SVM classifier testing
            p = Bev_performance(ds, dt)             # performance on test data


     D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """

    train, test, X, d, Xt, options = Bcl_construct(args)

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    options = options.copy()

    options['string'] = 'svm,{0}   '.format(options['kernel'])

    if train:

        all_kernels = ['linear', 'poly', 'rbf', 'sigmoid']
        k = 'rbf'
        c = options['kernel']
        if isinstance(c, int):
            if 0 <= c < len(all_kernels):
                k = all_kernels[c]
        elif c in all_kernels:
            k = c

        clf = svm.SVC()
        clf.fit(X, np.squeeze(d))
        options['svmStruct'] = clf
        output = options

    if test:
        clf = options['svmStruct']
        ds = clf.predict(Xt)[:, np.newaxis]
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import numpy as np
from .Bcl_construct import Bcl_construct
from .Bcl_outscore import Bcl_outscore


def Bcl_dmin(*args):
    """ ds, options = Bcl_dmin(X, d, Xt, None)  Training & Testing together
     options = Bcl_dmin(X, d, None)     Training only
     ds      = Bcl_dmin(Xt, options) Testing only

     Toolbox: Balu
        Classifier using Euclidean minimal distance

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data
           options['mc'] contains the centroids of each class.
           options[´dmin'] contains np.min(d).
           options['string'] is a 8 character string that describes the performed
           classification (in this case 'dmin    ').

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_dmin
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            ds, _ = Bcl_dmin(X, d, Xt, None)        # Euclidean distance classifier
            p = Bev_performance(ds, dt)             # performance on test data

        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_dmin

            data = balu_load('datagauss')           # simulated data (2 classes, 2 features)
            X = data['X']
            Xt = data['Xt']
            d = data['d']
            dt = data['dt']
            Bio_plotfeatures(X, d)                  # plot feature space
            op = Bcl_dmin(X, d, None)               # Euclidean distance classifier

        Example: Testing only (after training only example):
            ds, _ = Bcl_dmin(Xt, op)                   # Euclidean distance classifier - testing
            p = Bev_performance(ds, dt)             # performance on test data

        See also Bcl_maha.

     D.Mery, PUC-DCC, May 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    train, test, X, d, Xt, options = Bcl_construct(args)

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    if options is None:
        options = {}

    options = options.copy()

    options['string'] = 'dmin    '

    if train:
        m = X.shape[1]
        dmin = d.min()
        dmax = d.max()
        d = d - dmin + 1
        n = int(dmax - dmin + 1)
        mc = np.zeros((n, m))

        for i in range(int(n)):
            ii, _ = np.where(d == i + 1)
            mc[i, :] = np.mean(X[ii, :], axis=0)

        options['mc'] = mc
        options['dmin'] = dmin
        output = options

    if test:
        mc = options['mc']
        n = mc.shape[0]
        Nt = Xt.shape[0]
        ds = np.zeros((Nt, 1))
        sc = np.zeros((Nt, 1))

        for q in range(Nt):
            D = np.dot(np.ones((n, 1)), Xt[q, :][np.newaxis]) - mc
            e = np.sum(D * D, axis=1)
            i = np.min(e)
            j = np.argmin(e)
            ds[q, 0] = j
            sc[q, 0] = i

        ds = ds + options['dmin']
        ds = Bcl_outscore(ds, sc, options)
        output = ds, options

    return output



# -*- coding: utf-8 -*-
import numpy as np
import copy
from .Bcl_construct import Bcl_construct


def Bcl_structure(*args):
    """ ds = Bcl_structure(X, d, Xt, options)  Training & Testing together
     options = Bcl_structure(X, d, options)     Training only
     ds = Bcl_structure(Xt, options)      Testing only

     Toolbox: Balu
        Classification using Balu classifier(s) defined in structure b.

        Design data:
           X is a matrix with features (columns)
           d is the ideal classification for X
           options is a Balu classifier structure b with
              b.name      = Balu classifier's name
              b.options   = options of the classifier

           b can define one or more classifiers (see example).

        Test data:
           Xt is a matrix with features (columns)

        Output:
           ds is the classification on test data (one column per classifier)

        Example: Training & Test together:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_structure
            from balu.PerformanceEvaluation import Bev_performance

            data = balu_load('datagauss')                       # simulated data(2 classes, 2 features)
            X = data['X']
            d = data['d']
            Xt = data['Xt']
            dt = data['dt']
            Xn = ['\beta_1', '\beta_2']

            b = [
                {'name': 'knn',  'options': {'k': 5  }},         # KNN with 5 neighbors
                {'name': 'knn',  'options': {'k': 7  }},         # KNN with 5 neighbors
                {'name': 'knn',  'options': {'k': 9  }},         # KNN with 5 neighbors
                {'name': 'lda',  'options': {'p': [] }},         # LDA
                {'name': 'qda',  'options': {'p': [] }},         # QDA
                {'name': 'nn' ,  'options': {'method': 2}},      # Neural Network
                {'name': 'svm',  'options': {'kernel': 3}},      # rbf-SVM
                {'name': 'maha', 'options': {}},                 # Mahalanobis distance
                {'name': 'dmin', 'options': {}},                 # Euclidean distance
            ]
            op = b
            ds, struct = Bcl_structure(X, d, Xt, op)             # ds has 9 columns
            p = Bev_performance(ds, dt)                          # p has 9 performances


        Example: Training only
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_structure

            data = balu_load('datagauss')                       # simulated data(2 classes, 2 features)
            X = data['X']
            d = data['d']
            Xt = data['Xt']
            dt = data['dt']
            Xn = ['\beta_1', '\beta_2']

            b = [
                {'name': 'knn',  'options': {'k': 5  }},         # KNN with 5 neighbors
                {'name': 'knn',  'options': {'k': 7  }},         # KNN with 5 neighbors
                {'name': 'knn',  'options': {'k': 9  }},         # KNN with 5 neighbors
                {'name': 'lda',  'options': {'p': [] }},         # LDA
                {'name': 'qda',  'options': {'p': [] }},         # QDA
                {'name': 'nn' ,  'options': {'method': 2}},      # Neural Network
                {'name': 'svm',  'options': {'kernel': 3}},      # rbf-SVM
                {'name': 'maha', 'options': {}},                 # Mahalanobis distance
                {'name': 'dmin', 'options': {}},                 # Euclidean distance
            ]
            op = b
            struct = Bcl_structure(X, d, op)                     # Training only

        Example: Testing only (after training only example):
            ds, _ = Bcl_structure(Xt, struct)                    # Testing only
            p = Bev_performance(ds, dt)

        See also Bcl_exe.

     D.Mery, PUC-DCC, Jul 2009
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    train, test, X, d, Xt, options = Bcl_construct(args)
    options = copy.deepcopy(options)

    b = options
    if not isinstance(b, list):
        b = [b]

    n = len(b)                      # number of classifiers

    if train:
        for i in range(n):
            Bn = b[i]['name']
            if Bn[0] != 'B':
                Bn = 'Bcl_' + Bn

            cl = getattr(__import__('balu').Classification, Bn)
            b[i]['options'] = cl(X, d, b[i]['options'])

        options = b
        output = options

    if test:
        nt = Xt.shape[0]
        ds3 = np.zeros((nt, n, 2))
        d3 = 0
        for i in range(n):
            Bn = b[i]['name']
            if Bn[1] != 'B':
                Bn = 'Bcl_' + Bn

            cl = getattr(__import__('balu').Classification, Bn)
            dsi, _ = cl(Xt, b[i]['options'])

            if len(dsi.shape) > 1:
                ds3[:, i, 0] = dsi[:, 0]
                if dsi.shape[1] == 2:
                    ds3[:, i, 1] = dsi[:, 1]
                    d3 = True
            else:
                ds3[:, i, 0] = dsi

        if d3:
            ds = ds3
        else:
            ds = ds3[:, :, 0]

        output = ds, options

    return output



import numpy as np
from scipy.misc import imresize


def Bfx_fourier(I, *args):
    '''
    % [X,Xn,Xu] = Xfourier(I,R,options)
    % [X,Xn,Xu] = Xfourier(I,options)
    %
    % Toolbox Xvis: Fourier features
    %
    %    X is the features vector, Xn is the list of feature names (see Example
    %    to see how it works).
    %
    %   Example:
    %      options.Nfourier  = 64;                % imresize vertical
    %      options.Mfourier  = 64;                % imresize horizontal
    %      options.mfourier  = 2;                 % imresize frequency vertical
    %      options.nfourier  = 2;                 % imresize frequency horizontal
    %      options.show    = 1;                   % display results
    %      I = imread('testimg1.jpg');            % input image
    %      R = Bim_segbalu(I);                    % segmentation
    %      J = double(I(:,:,2))/256;              % normalized green channel
    %      [X,Xn] = Xfourier(J,R,options);     % Fourier features
    %      Bio_printfeatures(X,Xn)
    %
    %   See also Xharalick, Xclp, Xgabor, Xdct, Xlbp.
    %
    % (c) D.Mery, PUC-DCC, 2010
    % http://dmery.ing.puc.cl
    '''

    assert len(args) > 0, "Please provide at least the 'options' argument"

    if len(args) == 1:
        R = np.ones(I.shape)
        options = args[0]
    else:
        R = args[0]
        options = args[1]

    I[np.where(R == 0)] = 0

    N = options['Nfourier']
    M = options['Mfourier']
    n = options['nfourier']
    m = options['mfourier']

    N2 = np.round(N / 2.0)
    M2 = np.round(M / 2.0)

    if 'show' in options and options['show'] is True:
        print('--- extracting Fourier features...')

    Im = imresize(I.astype(float), (N, M))
    FIm = np.fft.fft2(Im)
    x = np.abs(FIm)
    F = imresize(x[0:N2, 0:M2], (n, m))
    x = np.angle(FIm)
    A = imresize(x[0:N2, 0:M2], (n, m))

    LS = 2 * n * m
    X = np.zeros((1, LS))
    Xn = LS * [None]
    k = -1
    for i in range(n):  # i=1:n
        for j in range(m):  # j=1:m
            k += 1
            # s = sprintf('Fourier Abs (%d,%d)              ', i, j)
            # Xn(k,:)  = s(1:24)
            Xn[k] = 'Fourier Abs ({}, {})'.format(i, j)
            X[0, k] = F[i, j]

    for i in range(n):
        for j in range(m):
            k += 1
            # s = sprintf('Fourier Ang (%d,%d)[rad]         ', i, j)
            # Xn(k,:)  = s(1:24)
            Xn[k] = 'Fourier Ang ({}, {})[rad]'.format(i, j)
            X[0, k] = A[i, j]

    return X, Xn

    '''if nargin==2;
        options = R;
        R = ones(size(I));
    end

    I(R==0) = 0;

    N  = options.Nfourier;
    M  = options.Mfourier;
    n  = options.nfourier;
    m  = options.mfourier;

    N2 = round(N/2);
    M2 = round(M/2);

    if options.show
        disp('--- extracting Fourier features...');
    end


    Im  = imresize(double(I),[N M]);
    FIm = fft2(Im);
    x   = abs(FIm);
    F   = imresize(x(1:N2,1:M2),[n m]);
    x   = angle(FIm);
    A   = imresize(x(1:N2,1:M2),[n m]);

    LS = 2*n*m;
    X  = zeros(1,LS);
    Xn = char(zeros(LS,24));
    k = 0;
    for i=1:n
        for j=1:m
            k = k + 1;
            s = sprintf('Fourier Abs (%d,%d)              ',i,j);
            Xn(k,:)  = s(1:24);
            X(k) = F(i,j);
        end
    end
    for i=1:n
        for j=1:m
            k = k + 1;
            s = sprintf('Fourier Ang (%d,%d)[rad]         ',i,j);
            Xn(k,:)  = s(1:24);
            X(k) = A(i,j);
        end
    end'''



# -*- coding: utf-8 -*-
import numpy as np
from skimage.morphology import closing, disk, remove_small_objects, remove_small_holes, erosion, square
from skimage.measure import perimeter


def Bim_morphoreg(J, t=None):
    """ R, E = Bmorphoreg(J, t)
     R, E = Bmorphoreg(Ro)

     Toolbox: Balu
        Morphology operations of binary image J>t (or Ro): remove isolate
        pixels and fill holes.
        R: binary image of the region
        E: binary image of the edge

      Example:
            from balu.ImagesAndData import balu_imageload
            from matplotlib.pyplot import figure, imshow, show
            from skimage.color import rgb2gray
            from balu.ImageProcessing import Bim_morphoreg, Bim_segotsu

            I = balu_imageload('testimg2.jpg')
            figure(1), imshow(I)
            J = rgb2gray(I.astype(float))
            Ro, E, J = Bim_segotsu(J)
            figure(2), imshow(Ro, cmap='gray')
            R, E = Bim_morphoreg(Ro)
            figure(3), imshow(R, cmap='gray')
            show()

     D.Mery, PUC-DCC, Apr. 2008
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if t is None:
        Ro = J
    else:
        Ro = J > t

    A = remove_small_objects(Ro, int(np.floor(Ro.size / 100.0)), connectivity=2)
    C = closing(A, disk(7))
    R = remove_small_holes(C, np.floor(Ro.size / 100), connectivity=2)
    E = R - erosion(R, selem=square(3))

    return R, E


# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
setup(
  name = 'balu',
  packages=find_packages(),
  version = '0.1.8',
  description = 'Python implementation of Balu Toolbox by Domingo Mery (http://dmery.ing.puc.cl/index.php/balu/)',
  author = 'Diego Patiño',
  author_email = 'dapatinoco@unal.edu.com',
  url = 'https://github.com/dipaco/balu-python',
  download_url = 'https://github.com/dipaco/balu-python/releases/tag/v0.1.0',
  keywords = ['Computer Vision', 'Machine Learning', 'Image Processing'],
  classifiers = [],
  install_requires = [
      'numpy>=1.11.2,<2',
      'scipy>=0.18.1,<1',
      'matplotlib>=1.5.2,<2',
      'scikit-image>=0.12.3,<1',
      'scikit-learn>=0.18,<1',
  ],
)



# -*- coding: utf-8 -*-


def Bfs_noposition(f, fn):
    """ f_new, fn_new = Bfs_noposition(f, fn)

     Toolbox: Balu
         This procedure deletes the features related to the position.
         It deletes the following features:
            - center of grav i
            - center of grav j
            - Ellipse-centre i
            - Ellipse-centre j

     Example:
            import numpy as np
            from balu.ImagesAndData import balu_imageload
            from balu.ImageProcessing import Bim_segbalu
            from balu.FeatureExtraction import Bfx_basicgeo, Bfx_fitellipse
            from balu.FeatureSelection import Bfs_noposition
            from balu.InputOutput import Bio_printfeatures

            I = balu_imageload('testimg1.jpg')              # input image
            R, E, J = Bim_segbalu(I)                        # segmentation
            X1, Xn1 = Bfx_basicgeo(R)                       # basic geometric features
            X2, Xn2 = Bfx_fitellipse(R)                     # Ellipse features
            X3 = np.hstack((X1, X2))
            Xn3 = Xn1 + Xn2
            print '\nOriginal features\n'
            Bio_printfeatures(X3, Xn3)
            X4, Xn4 = Bfs_noposition(X3, Xn3)               # delete position features
            print '\nSelected features\n'
            Bio_printfeatures(X4, Xn4)

      See also Bfs_norotation, Bfs_nobackground.

     D.Mery, PUC-DCC, Nov. 2009
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    ii = list(range(len(fn)))
    s = ['center of grav i [px]   ',
         'center of grav j [px]   ',
         'Ellipse-centre i [px]   ',
         'Ellipse-centre j [px]   ']
    fn_new = list(fn)

    for name in s:
        if name in fn:
            ii.remove(fn.index(name))
            fn_new.remove(name)

    if len(ii) > 0:
        f_new = f[:, ii]

    return f_new, fn_new



# -*- coding: utf-8 -*-
import numpy as np
from matplotlib.pyplot import show, clf, gca, close, figure, title, scatter
from .Bio_plotfeatures import Bio_plotfeatures
from balu.Classification import Bcl_structure


def Bio_decisionline(X, d, Xn, op):
    """ Bio_decisionline(X, d, Xn, op)

     Toolbox: Balu

        Diaplay a 2D feature space and decision line.

        X: Sample data
        d: classification of samples
        op: output of a trained classifier.

        Example:
            from balu.ImagesAndData import balu_load
            from balu.Classification import Bcl_structure
            from balu.InputOutput import Bio_decisionline
            from matplotlib.pyplot import close

            data = balu_load('datagauss')                       # simulated data(2 classes, 2 features)
            X = data['X']
            d = data['d']
            Xn = ['\beta_1', '\beta_2']

            b = [
                {'name': 'knn', 'options': {'k': 5  }},         # KNN with 5 neighbors
                {'name': 'lda', 'options': {'p': [] }},         # LDA
                {'name': 'qda', 'options': {'p': [] }},         # QDA
                {'name': 'svm', 'options': {'kernel': 3}},      # rbf-SVM
            ]
            op = b
            struct = Bcl_structure(X, d, op)
            close('all')
            Bio_decisionline(X, d, Xn, struct)

     (c) D.Mery, PUC-DCC, 2011
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
     """

    if len(d.shape) < 2:
        d = np.expand_dims(d, axis=1)

    if X.shape[1] != 2:
        print('Bio_decisionline works for two features only.')
        exit()

    clf()
    Bio_plotfeatures(X, d, Xn, hold=False)
    n = len(op)

    ax = gca()
    xl = ax.get_xlim()
    #x = xl[0]:s:xl[1]
    #x = np.linspace(xl[0], xl[1], int((xl[1] - xl[0]) / s))
    x = np.linspace(xl[0], xl[1], 150)
    nx = x.size
    yl = ax.get_ylim()
    #y = yl[0]:s:yl[1]
    #y = np.linspace(yl[0], yl[1], int((yl[1] - yl[0]) / s))
    y = np.linspace(yl[0], yl[1], 150)
    ny = y.size
    rx = np.dot(np.ones((ny, 1)), x[np.newaxis, :])
    ry = np.dot(y[:, np.newaxis], np.ones((1, nx)))
    XXt = np.vstack((rx.ravel(), ry.ravel())).T
    op = Bcl_structure(X, d, op)
    dt, _ = Bcl_structure(XXt, op)

    dmin = d.min().astype(int)
    dmax = d.max().astype(int)
    scol = 'ycwkbr'
    tcol = 'brgywk'

    close('all')
    for k in range(n):
        figure()
        Bio_plotfeatures(X, d, Xn, hold=False)
        title(op[k]['options']['string'])

        for i in range(dmin, dmax + 1):
            ii = np.where(dt[:, k] == i)[0]
            scatter(XXt[ii, 0], XXt[ii, 1], color=scol[i - dmin], marker='o')

        for i in range(dmin, dmax + 1):
            ii, jj = np.where(d == i)
            scatter(X[ii, 0], X[ii, 1], color=tcol[i - dmin], marker='x')

    show()



# -*- coding: utf-8 -*-
import numpy as np


def Bfx_all(I, R=None, options=None):
    """X, Xn = Bfx_all(I,R,options)
    X, Xn = Bfx_all(I,options)

    Toolbox: Balu
       All pixels.

       X is the features vector, Xn is the list feature names (see Example to
       see how it works).


       Example:
          I = imread('testimg1.jpg')             # input image
          X, Xn = Bfx_all(I)                   # all pixels


    (c) D.Mery, PUC-DCC, 2012
    http://dmery.ing.puc.cl

    With collaboration from:
    Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    X = np.zeros((1, I.size))
    X[0, :] = I.ravel()
    Xn = X.size * ['']
    return X, Xn



# -*- coding: utf-8 -*-
import numpy as np


def Bft_pca(X, m):
    """
     Y,lambda,A,Xs,mx = Bft_pca(X,m)

     Toolbox: Balu
        Principal component analysis
        X is the matrix feature.
        m number of selected components or the energy 0<m<=1 (in this case it
        will be selected the first d principal components that fulfill the
        condition sum(lambda(1:d))/sum(lambda) >= energy. See Example 2)
        Y contains the m principal components of X
        lambda contains the standard deviation of each principal component
        A is the transformation matrix from X to Y
        Xs is the reconstructed X from A and Y.

      Example 1:
        import numpy as np
        from balu.ImagesAndData import balu_load, balu_imageload
        from matplotlib.pyplot import figure, imshow, show, bar
        from balu.FeatureTransformation import Bft_pca

        X = balu_imageload('cameraman.tif').astype(float) # 256x256 pixels
        Y, l, A, Xs, mm = Bft_pca(X, 30)    # 30 principal components
        figure(1)
        ind = np.arange(l.size)
        bar(ind, l/l[0])
        figure(2)
        imshow(np.hstack((X, Xs.astype(float))), cmap='gray')
        show()

      Example 2:
        import numpy as np
        from balu.ImagesAndData import balu_load, balu_imageload
        from matplotlib.pyplot import figure, imshow, show, bar
        from balu.FeatureTransformation import Bft_pca

        X = balu_imageload('cameraman.tif').astype(float) # 256x256 pixels
        Y, l, A, Xs, mx = Bft_pca(X, 0.999)   # 90% of energy
        figure(1)
        ind = np.arange(l.size)
        bar(ind, l/l[0])
        figure(2)
        imshow(np.hstack((X, Xs.astype(float))), cmap='gray')
        show()

     (c) GRIMA, PUC-DCC, 2011
     http://grima.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    #function[Y, lambda , A, Xs, mx, B] = Bft_pca(X, d, options)

    N, L = X.shape

    if m > 0 and m < 1:
        energy = m
        m = L
    else:
        energy = 0

    mx = np.mean(X, axis=0)
    mx = np.expand_dims(mx, 1).T
    MX = np.dot(np.ones((N, 1)), mx)
    X0 = X - MX
    Cx = np.cov(X.T)
    lbd, A = np.linalg.eig(Cx)

    #In python the eigenvalues are not necessarily ordered
    idx = lbd.argsort()[::-1]
    lbd = lbd[idx]
    A = A[:, idx]

    if energy > 0:
        sumlambda = np.sum(lbd)
        energylam = np.zeros((L, 1))
        for i in range(0, L):
            energylam[i, 0] = sum(lbd[0:i+1]) / sumlambda
        ii, _ = np.where(energylam > energy)
        m = ii[0]

    B = A[:, 0:m]
    Y = np.dot(X0, B)   # Y = X0 * A
    Y = Y[:, 0:m]   # the first m components
    Xs = np.dot(Y, B.T) + MX

    return Y, lbd, A, Xs, mx



# -*- coding: utf-8 -*-
import numpy as np


def Bfx_geo(R, options):
    """ X, Xn = Bfx_geo(R, options)
     X, Xn = Bfx_geo(L, options)

     Toolbox: Balu

        Gemteric feature extraction.

        This function calls gemetric feature extraction procedures of binary
        image R or labelled image L.

        X is the feature matrix (one feature per column, one sample per row),
        Xn is the list with the names of these features (see Example
        to see how it works).

       Example 1: Extraction of one region image
            from balu.FeatureExtraction import Bfx_geo
            from balu.ImageProcessing import Bim_segbalu
            from balu.InputOutput import Bio_printfeatures
            from balu.ImagesAndData import balu_imageload

            options = {'b': [
                {'name': 'hugeo',      'options': {'show': True}},                      # Hu moments
                {'name': 'flusser',    'options': {'show': True}},                      # Flusser moments
                {'name': 'fourierdes', 'options': {'show': True, 'Nfourierdes': 12}}    # Fourier descriptors
            ]}

            I = balu_imageload('testimg1.jpg')                      # input image
            R = Bim_segbalu(I)                                      # segmentation
            X, Xn = Bfx_geo(R, options)                             # geometric features
            Bio_printfeatures(X, Xn)

       Example 2: Extraction of multiple regions image
            from balu.FeatureExtraction import Bfx_geo
            from balu.ImageProcessing import Bim_segmowgli
            from balu.ImagesAndData import balu_imageload
            from matplotlib.pyplot import figure, imshow, title, hist, xlabel, show

            options = {'b': [
                {'name': 'hugeo',      'options': {'show': True}},                      # Hu moments
                {'name': 'basicgeo',   'options': {'show': True}},                      # basic geometric features
                {'name': 'fourierdes', 'options': {'show': True, 'Nfourierdes': 12}}    # Fourier descriptors
            ]}

            I = balu_imageload('rice.png')                          # input image
            R, m = Bim_segmowgli(I, np.ones(I.shape), 40, 1.5)      # segmentation
            X, Xn = Bfx_geo(R, options)                             # geometric features
            figure(); hist(X[:, 11]); xlabel(Xn[11])                # area histogram
            ii = np.where(np.abs(X[:, 19]) < 15)[0]                 # rice orientation
            K = np.zeros(R.shape)                                   # between -15 and 15 grad
            for i in range(ii.size):
                K = np.logical_or(K, R == ii[i])
            figure(); imshow(K, cmap='gray'); title('abs(orientation)<15 grad')
            show()

       See also Bfx_basicgeo, Bfx_hugeo, Bfx_flusser, Bfx_gupta,
                Bfx_fitellipse, Bfx_fourierdes, Bfx_files.

     (c) D.Mery, PUC-DCC, 2010
     http://dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """
    if R.size == 0:
        print('Bfx_geo: R is empty. Geometric features without segmentation has no sense.')
        exit()
    else:
        b = options['b']
        n = len(b)
        m = int(R.max())
        X = np.array([])
        for j in range(1, m + 1):
            Rj = R==j
            Xj = np.array([])
            Xnj = []
            for i in range(n):
                s = b[i]['name']
                if s[0:3] != 'Bfx':
                    s = 'Bfx_' + s

                balu_module = __import__('balu')
                if s not in dir(balu_module.FeatureExtraction):
                    print('Bfx_geo: function {0} does not exist.'.format(b[i]['name']))
                    exit()

                f = getattr(balu_module.FeatureExtraction, s)
                Xi, Xni = f(Rj, b[i]['options'])
                Xj = np.hstack((Xj, np.squeeze(Xi)))
                Xnj += Xni

            if X.size == 0:
                X = np.hstack((X, Xj))
            else:
                X = np.vstack((X, Xj))

        if len(X.shape) == 1:
            X = np.expand_dims(X, axis=0)

        return X, Xnj


# -*- coding: utf-8 -*-
import numpy as np
from warnings import filterwarnings
from skimage.measure import regionprops, perimeter
from scipy.ndimage.morphology import distance_transform_cdt

def Bfx_basicgeo(R, *args):
    """X, Xn = Bfx_geobasic(R, options)

     Toolbox: Balu

     Standard geometric features of a binary image R.
     This function calls regionprops of Scikit image library.

     options['show'] = True display messages.

     X is the feature vector
     Xn is the list of feature names.

     See example:

     Example:
     from balu.ImagesAndData import balu_imageload
     I = balu_imageload('testimg1.jpg') #input image
     R = Bim_segbalu(I) # segmentation
     X, Xn = Bfx_basicgeo(R); # basic geometric features
     Bio_printfeatures(X, Xn)

     See also Bfx_fitellipse, Bfx_hugeo, Bfx_gupta, Bfx_flusser.

     (c) D.Mery, PUC - DCC, 2010
     http: // dmery.ing.puc.cl

     With collaboration from:
     Diego Patiño (dapatinoco@unal.edu.co) -> Translated implementation into python (2016)
    """

    if len(args) > 0:
        options = args[0]
    else:
        options = {'show': False}

    R = R.astype(bool)
    N = R.shape[0]
    filterwarnings('ignore')
    stats = regionprops(R.astype('uint8'))

    # Standard features
    # Perimeter
    if options['show']:
        print('--- extracting standard geometric features...')

    L8 = perimeter(R.astype('uint8'), neighbourhood=8)
    L4 = perimeter(R.astype('uint8'), neighbourhood=4)
    L = (3 * L4 + L8) / 4

    # Area
    A = stats[0].area

    # Roundness
    Roundness = 4 * A * np.pi / (L ** 2)

    # height & width
    [Ireg, Jreg] = np.where(R == 1)  # pixels in the region
    i_max = max(Ireg)
    i_min = min(Ireg)
    j_max = max(Jreg)
    j_min = min(Jreg)
    height = i_max - i_min + 1       # height
    width = j_max - j_min + 1        # width

    # Danielsson shape factor(see Danielsson, 1977)
    TD = distance_transform_cdt(R.astype('uint8'), metric='chessboard')
    dm = np.mean(TD[Ireg, Jreg])
    Gd = A / 9 / np.pi / dm ** 2

    X = np.array([[
        stats[0].centroid[0],
        stats[0].centroid[1],
        height,
        width,
        A,
        L,
        Roundness,
        Gd,
        stats[0].euler_number,
        stats[0].equivalent_diameter,
        stats[0].major_axis_length,
        stats[0].minor_axis_length,
        stats[0].orientation * 180.0 / np.pi,
        stats[0].solidity,
        stats[0].extent,
        stats[0].eccentricity,
        stats[0].convex_area,
        stats[0].filled_area]]
    );

    Xn = [
        'center of grav i [px]   ',
        'center of grav j [px]   ',
        'Height [px]             ',
        'Width [px]              ',
        'Area [px]               ',
        'Perimeter [px]          ',
        'Roundness               ',
        'Danielsson factor       ',
        'Euler Number            ',
        'Equivalent Diameter [px]',
        'MajorAxisLength [px]    ',
        'MinorAxisLength [px]    ',
        'Orientation [grad]      ',
        'Solidity                ',
        'Extent                  ',
        'Eccentricity            ',
        'Convex Area [px]        ',
        'Filled Area [px]        ']
    return X, Xn


# -*- coding: utf-8 -*-
import numpy as np
from skimage.feature import local_binary_pattern
from scipy.stats import itemfreq


def Bfx_lbp(I, R, options):

    if 'show' not in options:
        options['show'] = False

    if 'normalize' not in options:
        options['normalize'] = False

    if options['show']:
        print('--- extracting local binary patterns features...')

    if 'samples' not in options:
        options['samples'] = 8

    if 'integral' not in options:
        options['integral'] = 0

    if 'radius' not in options:
        options['radius'] = np.log(options['samples']) / np.log(2.0) - 1

    if 'semantic' not in options:
        options['semantic'] = 0

    if 'weight' not in options:
        options['weight'] = 0

    P = options['samples']
    R = options['radius']
    LBP = local_binary_pattern(I, P=P, R=2, method='nri_uniform')
    print(LBP.max())
    X = (np.histogram(LBP[2:-2, 2:-2].ravel(), 59)[0])[None]
    #X = itemfreq(LBP)
    if options['normalize']:
        X /= float(X.sum())

    #1 LBP(1,1)[8,u2]
    Xn = ['{0} LBP(1,{1})[{2},{3}]'.format(i, i, P, options['mappingtype']) for i in range(X.shape[1])]

    return X, Xn